"""
Python module generated from Java source file org.bukkit.entity.AbstractArrow

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block import Block
from org.bukkit.entity import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class AbstractArrow(Projectile):
    """
    Represents an arrow.
    """

    def getKnockbackStrength(self) -> int:
        """
        Gets the knockback strength for an arrow, which is the
        org.bukkit.enchantments.Enchantment.KNOCKBACK KnockBack level
        of the bow that shot it.

        Returns
        - the knockback strength value

        See
        - .getWeapon()

        Deprecated
        - a function of the firing weapon
        """
        ...


    def setKnockbackStrength(self, knockbackStrength: int) -> None:
        """
        Sets the knockback strength for an arrow.

        Arguments
        - knockbackStrength: the knockback strength value

        See
        - .setWeapon(org.bukkit.inventory.ItemStack)

        Deprecated
        - a function of the firing weapon
        """
        ...


    def getDamage(self) -> float:
        """
        Gets the base amount of damage this arrow will do.
        
        Defaults to 2.0 for a normal arrow with
        `0.5 * (1 + power level)` added for arrows fired from
        enchanted bows.

        Returns
        - base damage amount
        """
        ...


    def setDamage(self, damage: float) -> None:
        """
        Sets the base amount of damage this arrow will do.

        Arguments
        - damage: new damage amount
        """
        ...


    def getPierceLevel(self) -> int:
        """
        Gets the number of times this arrow can pierce through an entity.

        Returns
        - pierce level
        """
        ...


    def setPierceLevel(self, pierceLevel: int) -> None:
        """
        Sets the number of times this arrow can pierce through an entity.
        
        Must be between 0 and 127 times.

        Arguments
        - pierceLevel: new pierce level
        """
        ...


    def isCritical(self) -> bool:
        """
        Gets whether this arrow is critical.
        
        Critical arrows have increased damage and cause particle effects.
        
        Critical arrows generally occur when a player fully draws a bow before
        firing.

        Returns
        - True if it is critical
        """
        ...


    def setCritical(self, critical: bool) -> None:
        """
        Sets whether or not this arrow should be critical.

        Arguments
        - critical: whether or not it should be critical
        """
        ...


    def isInBlock(self) -> bool:
        """
        Gets whether this arrow is in a block or not.
        
        Arrows in a block are motionless and may be picked up by players.

        Returns
        - True if in a block
        """
        ...


    def getAttachedBlock(self) -> "Block":
        """
        Gets the block to which this arrow is attached.

        Returns
        - the attached block or null if not attached
        """
        ...


    def getPickupStatus(self) -> "PickupStatus":
        """
        Gets the current pickup status of this arrow.

        Returns
        - the pickup status of this arrow.
        """
        ...


    def setPickupStatus(self, status: "PickupStatus") -> None:
        """
        Sets the current pickup status of this arrow.

        Arguments
        - status: new pickup status of this arrow.
        """
        ...


    def isShotFromCrossbow(self) -> bool:
        """
        Gets if this arrow was shot from a crossbow.

        Returns
        - if shot from a crossbow
        """
        ...


    def setShotFromCrossbow(self, shotFromCrossbow: bool) -> None:
        """
        Sets if this arrow was shot from a crossbow.

        Arguments
        - shotFromCrossbow: if shot from a crossbow

        See
        - .setWeapon(org.bukkit.inventory.ItemStack)

        Deprecated
        - a function of the firing weapon instead
        """
        ...


    def getItem(self) -> "ItemStack":
        """
        Gets the ItemStack which will be picked up from this arrow.

        Returns
        - The picked up ItemStack
        """
        ...


    def setItem(self, item: "ItemStack") -> None:
        """
        Sets the ItemStack which will be picked up from this arrow.

        Arguments
        - item: ItemStack set to be picked up
        """
        ...


    def getWeapon(self) -> "ItemStack":
        """
        Gets the ItemStack which fired this arrow.

        Returns
        - The firing ItemStack
        """
        ...


    def setWeapon(self, item: "ItemStack") -> None:
        """
        Sets the ItemStack which fired this arrow.

        Arguments
        - item: The firing ItemStack
        """
        ...


    class PickupStatus(Enum):
        """
        Represents the pickup status of this arrow.
        """

        DISALLOWED = 0
        """
        The arrow cannot be picked up.
        """
        ALLOWED = 1
        """
        The arrow can be picked up.
        """
        CREATIVE_ONLY = 2
        """
        The arrow can only be picked up by players in creative mode.
        """



"""
Python module generated from Java source file org.bukkit.Keyed

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class Keyed:
    """
    Represents an object which has a NamespacedKey attached to it.
    """

    def getKey(self) -> "NamespacedKey":
        """
        Return the namespaced identifier for this object.

        Returns
        - this object's key
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.inventory.InventoryType

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.event.inventory import *
from org.bukkit.inventory import InventoryHolder
from org.bukkit.inventory import MenuType
from typing import Any, Callable, Iterable, Tuple


class InventoryType(Enum):
    """
    Represents the different kinds of inventories available in Bukkit.
    
    Only InventoryTypes marked .isCreatable() can be created.
    
    The current list of inventories that cannot be created via
    org.bukkit.Bukkit.createInventory are:
    <blockquote>
        InventoryType.CREATIVE, InventoryType.CRAFTING and
        InventoryType.MERCHANT
    </blockquote>
    
    See org.bukkit.Bukkit.createInventory for more information.

    See
    - org.bukkit.Bukkit.createInventory(InventoryHolder, InventoryType)
    """

    CHEST = (27, "Chest", MenuType.GENERIC_9X3)
    """
    A chest inventory, with 0, 9, 18, 27, 36, 45, or 54 slots of type
    CONTAINER.
    """
    DISPENSER = (9, "Dispenser", MenuType.GENERIC_3X3)
    """
    A dispenser inventory, with 9 slots of type CONTAINER.
    """
    DROPPER = (9, "Dropper", MenuType.GENERIC_3X3)
    """
    A dropper inventory, with 9 slots of type CONTAINER.
    """
    FURNACE = (3, "Furnace", MenuType.FURNACE)
    """
    A furnace inventory, with a RESULT slot, a CRAFTING slot, and a FUEL
    slot.
    """
    WORKBENCH = (10, "Crafting", MenuType.CRAFTING)
    """
    A workbench inventory, with 9 CRAFTING slots and a RESULT slot.
    """
    CRAFTING = (5, "Crafting", None, False)
    """
    A player's crafting inventory, with 4 CRAFTING slots and a RESULT slot.
    Also implies that the 4 ARMOR slots are accessible.
    """
    ENCHANTING = (2, "Enchanting", MenuType.ENCHANTMENT)
    """
    An enchantment table inventory, with two CRAFTING slots and three
    enchanting buttons.
    """
    BREWING = (5, "Brewing", MenuType.BREWING_STAND)
    """
    A brewing stand inventory, with one FUEL slot and four CRAFTING slots.
    """
    PLAYER = (41, "Player", MenuType.GENERIC_9X4)
    """
    A player's inventory, with 9 QUICKBAR slots, 27 CONTAINER slots, 4 ARMOR
    slots and 1 offhand slot. The ARMOR and offhand slots may not be visible
    to the player, though.
    """
    CREATIVE = (9, "Creative", None, False)
    """
    The creative mode inventory, with only 9 QUICKBAR slots and nothing
    else. (The actual creative interface with the items is client-side and
    cannot be altered by the server.)
    """
    MERCHANT = (3, "Villager", MenuType.MERCHANT, False)
    """
    The merchant inventory, with 2 CRAFTING slots, and 1 RESULT slot.
    """
    ENDER_CHEST = (27, "Ender Chest", MenuType.GENERIC_9X3)
    """
    The ender chest inventory, with 27 slots.
    """
    ANVIL = (3, "Repairing", MenuType.ANVIL)
    """
    An anvil inventory, with 2 CRAFTING slots and 1 RESULT slot
    """
    SMITHING = (4, "Upgrade Gear", MenuType.SMITHING)
    """
    A smithing inventory, with 3 CRAFTING slots and 1 RESULT slot.
    """
    BEACON = (1, "container.beacon", MenuType.BEACON)
    """
    A beacon inventory, with 1 CRAFTING slot
    """
    HOPPER = (5, "Item Hopper", MenuType.HOPPER)
    """
    A hopper inventory, with 5 slots of type CONTAINER.
    """
    SHULKER_BOX = (27, "Shulker Box", MenuType.SHULKER_BOX)
    """
    A shulker box inventory, with 27 slots of type CONTAINER.
    """
    BARREL = (27, "Barrel", MenuType.GENERIC_9X3)
    """
    A barrel box inventory, with 27 slots of type CONTAINER.
    """
    BLAST_FURNACE = (3, "Blast Furnace", MenuType.BLAST_FURNACE)
    """
    A blast furnace inventory, with a RESULT slot, a CRAFTING slot, and a
    FUEL slot.
    """
    LECTERN = (1, "Lectern", MenuType.LECTERN)
    """
    A lectern inventory, with 1 BOOK slot.
    """
    SMOKER = (3, "Smoker", MenuType.SMOKER)
    """
    A smoker inventory, with a RESULT slot, a CRAFTING slot, and a FUEL slot.
    """
    LOOM = (4, "Loom", MenuType.LOOM)
    """
    Loom inventory, with 3 CRAFTING slots, and 1 RESULT slot.
    """
    CARTOGRAPHY = (3, "Cartography Table", MenuType.CARTOGRAPHY_TABLE)
    """
    Cartography inventory with 2 CRAFTING slots, and 1 RESULT slot.
    """
    GRINDSTONE = (3, "Repair & Disenchant", MenuType.GRINDSTONE)
    """
    Grindstone inventory with 2 CRAFTING slots, and 1 RESULT slot.
    """
    STONECUTTER = (2, "Stonecutter", MenuType.STONECUTTER)
    """
    Stonecutter inventory with 1 CRAFTING slot, and 1 RESULT slot.
    """
    COMPOSTER = (1, "Composter", None, False)
    """
    Pseudo composter inventory with 0 or 1 slots of undefined type.
    """
    CHISELED_BOOKSHELF = (6, "Chiseled Bookshelf", None, False)
    """
    Pseudo chiseled bookshelf inventory, with 6 slots of undefined type.
    """
    JUKEBOX = (1, "Jukebox", None, False)
    """
    Pseudo jukebox inventory with 1 slot of undefined type.
    """
    CRAFTER = (9, "Crafter", MenuType.CRAFTER_3X3)
    """
    A crafter inventory, with 9 CRAFTING slots.
    """
    SMITHING_NEW = (4, "Upgrade Gear", MenuType.SMITHING)
    """
    The new smithing inventory, with 3 CRAFTING slots and 1 RESULT slot.

    Deprecated
    - use .SMITHING
    """


    def getDefaultSize(self) -> int:
        ...


    def getDefaultTitle(self) -> str:
        ...


    def getMenuType(self) -> "MenuType":
        """
        Gets the corresponding MenuType of this InventoryType.
        
        Not all InventoryType correspond to a MenuType. These
        InventoryTypes are also not creatable. If this method returns null,
        .isCreatable() will return False, with the exception of
        .MERCHANT.
        
        As well as not necessarily corresponding to a MenuType some
        InventoryType correspond to the same MenuType, including:
        
        - Dropper, Dispenser
        - ShulkerBox, Barrel, Chest

        Returns
        - the corresponding MenuType
        """
        ...


    def isCreatable(self) -> bool:
        """
        Denotes that this InventoryType can be created via the normal
        org.bukkit.Bukkit.createInventory methods.

        Returns
        - if this InventoryType can be created and shown to a player
        """
        ...


    class SlotType(Enum):

        RESULT = 0
        """
        A result slot in a furnace or crafting inventory.
        """
        CRAFTING = 1
        """
        A slot in the crafting matrix, or an 'input' slot.
        """
        ARMOR = 2
        """
        An armour slot in the player's inventory.
        """
        CONTAINER = 3
        """
        A regular slot in the container or the player's inventory; anything
        not covered by the other enum values.
        """
        QUICKBAR = 4
        """
        A slot in the bottom row or quickbar.
        """
        OUTSIDE = 5
        """
        A pseudo-slot representing the area outside the inventory window.
        """
        FUEL = 6
        """
        The fuel slot in a furnace inventory, or the ingredient slot in a
        brewing stand inventory.
        """



from .Mark import Mark
from .MissingEnvironmentVariableException import MissingEnvironmentVariableException
from .MarkedYAMLException import MarkedYAMLException
from .YAMLException import YAMLException


"""
Python module generated from Java source file org.yaml.snakeyaml.resolver.ResolverTuple

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.regex import Pattern
from org.yaml.snakeyaml.nodes import Tag
from org.yaml.snakeyaml.resolver import *
from typing import Any, Callable, Iterable, Tuple


class ResolverTuple:

    def __init__(self, tag: "Tag", regexp: "Pattern", limit: int):
        ...


    def getTag(self) -> "Tag":
        ...


    def getRegexp(self) -> "Pattern":
        ...


    def getLimit(self) -> int:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.components.FoodComponent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.configuration.serialization import ConfigurationSerializable
from org.bukkit.inventory.meta.components import *
from org.bukkit.inventory.meta.components.consumable import ConsumableComponent
from typing import Any, Callable, Iterable, Tuple


class FoodComponent(ConfigurationSerializable):
    """
    Represents a component which can handle food stats in any item.
    
    **Note:** Items with food stats has no effect unless the item can be
    consumed, see ConsumableComponent.
    """

    def getNutrition(self) -> int:
        """
        Gets the food restored by this item when eaten.

        Returns
        - nutrition value
        """
        ...


    def setNutrition(self, nutrition: int) -> None:
        """
        Sets the food restored by this item when eaten.

        Arguments
        - nutrition: new nutrition value, must be non-negative
        """
        ...


    def getSaturation(self) -> float:
        """
        Gets the saturation restored by this item when eaten.

        Returns
        - saturation value
        """
        ...


    def setSaturation(self, saturation: float) -> None:
        """
        Sets the saturation restored by this item when eaten.

        Arguments
        - saturation: new saturation value
        """
        ...


    def canAlwaysEat(self) -> bool:
        """
        Gets if this item can be eaten even when not hungry.

        Returns
        - True if always edible
        """
        ...


    def setCanAlwaysEat(self, canAlwaysEat: bool) -> None:
        """
        Sets if this item can be eaten even when not hungry.

        Arguments
        - canAlwaysEat: whether always edible
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.SingletonImmutableList

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Preconditions
from com.google.common.collect import *
from java.util import Collections
from java.util import Spliterator
from typing import Any, Callable, Iterable, Tuple


class SingletonImmutableList(ImmutableList):
    """
    Implementation of ImmutableList with exactly one element.

    Author(s)
    - Hayward Chan
    """

    def get(self, index: int) -> "E":
        ...


    def iterator(self) -> "UnmodifiableIterator"["E"]:
        ...


    def spliterator(self) -> "Spliterator"["E"]:
        ...


    def size(self) -> int:
        ...


    def subList(self, fromIndex: int, toIndex: int) -> "ImmutableList"["E"]:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file java.io.BufferedOutputStream

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from typing import Any, Callable, Iterable, Tuple


class BufferedOutputStream(FilterOutputStream):
    """
    The class implements a buffered output stream. By setting up such
    an output stream, an application can write bytes to the underlying
    output stream without necessarily causing a call to the underlying
    system for each byte written.

    Author(s)
    - Arthur van Hoff

    Since
    - 1.0
    """

    def __init__(self, out: "OutputStream"):
        """
        Creates a new buffered output stream to write data to the
        specified underlying output stream.

        Arguments
        - out: the underlying output stream.
        """
        ...


    def __init__(self, out: "OutputStream", size: int):
        """
        Creates a new buffered output stream to write data to the
        specified underlying output stream with the specified buffer
        size.

        Arguments
        - out: the underlying output stream.
        - size: the buffer size.

        Raises
        - IllegalArgumentException: if size &lt;= 0.
        """
        ...


    def write(self, b: int) -> None:
        """
        Writes the specified byte to this buffered output stream.

        Arguments
        - b: the byte to be written.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def write(self, b: list[int], off: int, len: int) -> None:
        """
        Writes `len` bytes from the specified byte array
        starting at offset `off` to this buffered output stream.
        
         Ordinarily this method stores bytes from the given array into this
        stream's buffer, flushing the buffer to the underlying output stream as
        needed.  If the requested length is at least as large as this stream's
        buffer, however, then this method will flush the buffer and write the
        bytes directly to the underlying output stream.  Thus redundant
        `BufferedOutputStream`s will not copy data unnecessarily.

        Arguments
        - b: the data.
        - off: the start offset in the data.
        - len: the number of bytes to write.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def flush(self) -> None:
        """
        Flushes this buffered output stream. This forces any buffered
        output bytes to be written out to the underlying output stream.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterOutputStream.out
        """
        ...



"""
Python module generated from Java source file com.google.common.graph.EdgesConnecting

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import ImmutableSet
from com.google.common.collect import Iterators
from com.google.common.collect import UnmodifiableIterator
from com.google.common.graph import *
from java.util import AbstractSet
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class EdgesConnecting(AbstractSet):
    """
    A class to represent the set of edges connecting an (implicit) origin node to a target node.
    
    The .nodeToOutEdge map means this class only works on networks without parallel edges.
    See MultiEdgesConnecting for a class that works with parallel edges.
    
    Type `<E>`: Edge parameter type

    Author(s)
    - James Sexton
    """

    def iterator(self) -> "UnmodifiableIterator"["E"]:
        ...


    def size(self) -> int:
        ...


    def contains(self, edge: "Object") -> bool:
        ...



"""
Python module generated from Java source file com.google.common.collect.DiscreteDomain

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.common.primitives import Ints
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.io import Serializable
from java.math import BigInteger
from java.util import NoSuchElementException
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class DiscreteDomain:
    """
    A descriptor for a *discrete* `Comparable` domain such as all Integer
    instances. A discrete domain is one that supports the three basic operations: .next,
    .previous and .distance, according to their specifications. The methods .minValue and .maxValue should also be overridden for bounded types.
    
    A discrete domain always represents the *entire* set of values of its type; it cannot
    represent partial domains such as "prime integers" or "strings of length 5."
    
    See the Guava User Guide section on <a href=
    "https://github.com/google/guava/wiki/RangesExplained#discrete-domains">`DiscreteDomain`</a>.

    Author(s)
    - Kevin Bourrillion

    Since
    - 10.0
    """

    @staticmethod
    def integers() -> "DiscreteDomain"["Integer"]:
        """
        Returns the discrete domain for values of type `Integer`.
        
        This method always returns the same object. That object is serializable; deserializing it
        results in the same object too.

        Since
        - 14.0 (since 10.0 as `DiscreteDomains.integers()`)
        """
        ...


    @staticmethod
    def longs() -> "DiscreteDomain"["Long"]:
        """
        Returns the discrete domain for values of type `Long`.
        
        This method always returns the same object. That object is serializable; deserializing it
        results in the same object too.

        Since
        - 14.0 (since 10.0 as `DiscreteDomains.longs()`)
        """
        ...


    @staticmethod
    def bigIntegers() -> "DiscreteDomain"["BigInteger"]:
        """
        Returns the discrete domain for values of type `BigInteger`.
        
        This method always returns the same object. That object is serializable; deserializing it
        results in the same object too.

        Since
        - 15.0
        """
        ...


    def next(self, value: "C") -> "C":
        """
        Returns the unique least value of type `C` that is greater than `value`, or `null` if none exists. Inverse operation to .previous.

        Arguments
        - value: any value of type `C`

        Returns
        - the least value greater than `value`, or `null` if `value` is `maxValue()`
        """
        ...


    def previous(self, value: "C") -> "C":
        """
        Returns the unique greatest value of type `C` that is less than `value`, or `null` if none exists. Inverse operation to .next.

        Arguments
        - value: any value of type `C`

        Returns
        - the greatest value less than `value`, or `null` if `value` is `minValue()`
        """
        ...


    def distance(self, start: "C", end: "C") -> int:
        """
        Returns a signed value indicating how many nested invocations of .next (if positive) or
        .previous (if negative) are needed to reach `end` starting from `start`.
        For example, if `end = next(next(next(start)))`, then `distance(start, end) == 3`
        and `distance(end, start) == -3`. As well, `distance(a, a)` is always zero.
        
        Note that this function is necessarily well-defined for any discrete type.

        Returns
        - the distance as described above, or Long.MIN_VALUE or Long.MAX_VALUE if
            the distance is too small or too large, respectively.
        """
        ...


    def minValue(self) -> "C":
        """
        Returns the minimum value of type `C`, if it has one. The minimum value is the unique
        value for which Comparable.compareTo(Object) never returns a positive value for any
        input of type `C`.
        
        The default implementation throws `NoSuchElementException`.

        Returns
        - the minimum value of type `C`; never null

        Raises
        - NoSuchElementException: if the type has no (practical) minimum value; for example,
            java.math.BigInteger
        """
        ...


    def maxValue(self) -> "C":
        """
        Returns the maximum value of type `C`, if it has one. The maximum value is the unique
        value for which Comparable.compareTo(Object) never returns a negative value for any
        input of type `C`.
        
        The default implementation throws `NoSuchElementException`.

        Returns
        - the maximum value of type `C`; never null

        Raises
        - NoSuchElementException: if the type has no (practical) maximum value; for example,
            java.math.BigInteger
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.Levelled

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block.data import *
from typing import Any, Callable, Iterable, Tuple


class Levelled(BlockData):
    """
    'level' represents the amount of fluid contained within this block, either by
    itself or inside a cauldron.
    
    In the case of water and lava blocks the levels have special meanings: a
    level of 0 corresponds to a source block, 1-7 regular fluid heights, and 8-15
    to "falling" fluids. All falling fluids have the same behaviour, but the
    level corresponds to that of the block above them, equal to
    `this.level - 8`
    **Note that counterintuitively, an adjusted level of 1 is the highest level,
    whilst 7 is the lowest.**
    
    May not be higher than .getMaximumLevel().
    """

    def getLevel(self) -> int:
        """
        Gets the value of the 'level' property.

        Returns
        - the 'level' value
        """
        ...


    def setLevel(self, level: int) -> None:
        """
        Sets the value of the 'level' property.

        Arguments
        - level: the new 'level' value
        """
        ...


    def getMaximumLevel(self) -> int:
        """
        Gets the maximum allowed value of the 'level' property.

        Returns
        - the maximum 'level' value
        """
        ...



"""
Python module generated from Java source file java.net.Inet4Address

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import ObjectStreamException
from java.net import *
from typing import Any, Callable, Iterable, Tuple


class Inet4Address(InetAddress):

    def isMulticastAddress(self) -> bool:
        """
        Utility routine to check if the InetAddress is an
        IP multicast address. IP multicast address is a Class D
        address i.e first four bits of the address are 1110.

        Returns
        - a `boolean` indicating if the InetAddress is
        an IP multicast address
        """
        ...


    def isAnyLocalAddress(self) -> bool:
        """
        Utility routine to check if the InetAddress is a wildcard address.

        Returns
        - a `boolean` indicating if the InetAddress is
                a wildcard address.
        """
        ...


    def isLoopbackAddress(self) -> bool:
        """
        Utility routine to check if the InetAddress is a loopback address.

        Returns
        - a `boolean` indicating if the InetAddress is
        a loopback address; or False otherwise.
        """
        ...


    def isLinkLocalAddress(self) -> bool:
        """
        Utility routine to check if the InetAddress is an link local address.

        Returns
        - a `boolean` indicating if the InetAddress is
        a link local address; or False if address is not a link local unicast address.
        """
        ...


    def isSiteLocalAddress(self) -> bool:
        """
        Utility routine to check if the InetAddress is a site local address.

        Returns
        - a `boolean` indicating if the InetAddress is
        a site local address; or False if address is not a site local unicast address.
        """
        ...


    def isMCGlobal(self) -> bool:
        """
        Utility routine to check if the multicast address has global scope.

        Returns
        - a `boolean` indicating if the address has
                is a multicast address of global scope, False if it is not
                of global scope or it is not a multicast address
        """
        ...


    def isMCNodeLocal(self) -> bool:
        """
        Utility routine to check if the multicast address has node scope.

        Returns
        - a `boolean` indicating if the address has
                is a multicast address of node-local scope, False if it is not
                of node-local scope or it is not a multicast address
        """
        ...


    def isMCLinkLocal(self) -> bool:
        """
        Utility routine to check if the multicast address has link scope.

        Returns
        - a `boolean` indicating if the address has
                is a multicast address of link-local scope, False if it is not
                of link-local scope or it is not a multicast address
        """
        ...


    def isMCSiteLocal(self) -> bool:
        """
        Utility routine to check if the multicast address has site scope.

        Returns
        - a `boolean` indicating if the address has
                is a multicast address of site-local scope, False if it is not
                of site-local scope or it is not a multicast address
        """
        ...


    def isMCOrgLocal(self) -> bool:
        """
        Utility routine to check if the multicast address has organization scope.

        Returns
        - a `boolean` indicating if the address has
                is a multicast address of organization-local scope,
                False if it is not of organization-local scope
                or it is not a multicast address
        """
        ...


    def getAddress(self) -> list[int]:
        """
        Returns the raw IP address of this `InetAddress`
        object. The result is in network byte order: the highest order
        byte of the address is in `getAddress()[0]`.

        Returns
        - the raw IP address of this object.
        """
        ...


    def getHostAddress(self) -> str:
        """
        Returns the IP address string in textual presentation form.

        Returns
        - the raw IP address in a string format.
        """
        ...


    def hashCode(self) -> int:
        """
        Returns a hashcode for this IP address.

        Returns
        - a hash code value for this IP address.
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Compares this object against the specified object.
        The result is `True` if and only if the argument is
        not `null` and it represents the same IP address as
        this object.
        
        Two instances of `InetAddress` represent the same IP
        address if the length of the byte arrays returned by
        `getAddress` is the same for both, and each of the
        array components is the same for the byte arrays.

        Arguments
        - obj: the object to compare against.

        Returns
        - `True` if the objects are the same;
                 `False` otherwise.

        See
        - java.net.InetAddress.getAddress()
        """
        ...



"""
Python module generated from Java source file java.lang.reflect.Array

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.reflect import *
from jdk.internal.vm.annotation import IntrinsicCandidate
from typing import Any, Callable, Iterable, Tuple


class Array:
    """
    The `Array` class provides static methods to dynamically create and
    access Java arrays.
    
    `Array` permits widening conversions to occur during a get or set
    operation, but throws an `IllegalArgumentException` if a narrowing
    conversion would occur.

    Author(s)
    - Nakul Saraiya

    Since
    - 1.1
    """

    @staticmethod
    def newInstance(componentType: type[Any], length: int) -> "Object":
        """
        Creates a new array with the specified component type and
        length.
        Invoking this method is equivalent to creating an array
        as follows:
        <blockquote>
        ```
        int[] x = {length};
        Array.newInstance(componentType, x);
        ```
        </blockquote>
        
        The number of dimensions of the new array must not
        exceed 255.

        Arguments
        - componentType: the `Class` object representing the
                component type of the new array
        - length: the length of the new array

        Returns
        - the new array

        Raises
        - NullPointerException: if the specified
                `componentType` parameter is null
        - IllegalArgumentException: if componentType is Void.TYPE or if the number of dimensions of the requested array
                instance exceed 255.
        - NegativeArraySizeException: if the specified `length`
                is negative
        """
        ...


    @staticmethod
    def newInstance(componentType: type[Any], *dimensions: Tuple[int, ...]) -> "Object":
        """
        Creates a new array
        with the specified component type and dimensions.
        If `componentType`
        represents a non-array class or interface, the new array
        has `dimensions.length` dimensions and
        `componentType` as its component type. If
        `componentType` represents an array class, the
        number of dimensions of the new array is equal to the sum
        of `dimensions.length` and the number of
        dimensions of `componentType`. In this case, the
        component type of the new array is the component type of
        `componentType`.
        
        The number of dimensions of the new array must not
        exceed 255.

        Arguments
        - componentType: the `Class` object representing the component
        type of the new array
        - dimensions: an array of `int` representing the dimensions of
        the new array

        Returns
        - the new array

        Raises
        - NullPointerException: if the specified
        `componentType` argument is null
        - IllegalArgumentException: if the specified `dimensions`
        argument is a zero-dimensional array, if componentType is Void.TYPE, or if the number of dimensions of the requested array
        instance exceed 255.
        - NegativeArraySizeException: if any of the components in
        the specified `dimensions` argument is negative.
        """
        ...


    @staticmethod
    def getLength(array: "Object") -> int:
        """
        Returns the length of the specified array object, as an `int`.

        Arguments
        - array: the array

        Returns
        - the length of the array

        Raises
        - IllegalArgumentException: if the object argument is not
        an array
        """
        ...


    @staticmethod
    def get(array: "Object", index: int) -> "Object":
        """
        Returns the value of the indexed component in the specified
        array object.  The value is automatically wrapped in an object
        if it has a primitive type.

        Arguments
        - array: the array
        - index: the index

        Returns
        - the (possibly wrapped) value of the indexed component in
        the specified array

        Raises
        - NullPointerException: If the specified object is null
        - IllegalArgumentException: If the specified object is not
        an array
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to the
        length of the specified array
        """
        ...


    @staticmethod
    def getBoolean(array: "Object", index: int) -> bool:
        """
        Returns the value of the indexed component in the specified
        array object, as a `boolean`.

        Arguments
        - array: the array
        - index: the index

        Returns
        - the value of the indexed component in the specified array

        Raises
        - NullPointerException: If the specified object is null
        - IllegalArgumentException: If the specified object is not
        an array, or if the indexed element cannot be converted to the
        return type by an identity or widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to the
        length of the specified array

        See
        - Array.get
        """
        ...


    @staticmethod
    def getByte(array: "Object", index: int) -> int:
        """
        Returns the value of the indexed component in the specified
        array object, as a `byte`.

        Arguments
        - array: the array
        - index: the index

        Returns
        - the value of the indexed component in the specified array

        Raises
        - NullPointerException: If the specified object is null
        - IllegalArgumentException: If the specified object is not
        an array, or if the indexed element cannot be converted to the
        return type by an identity or widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to the
        length of the specified array

        See
        - Array.get
        """
        ...


    @staticmethod
    def getChar(array: "Object", index: int) -> str:
        """
        Returns the value of the indexed component in the specified
        array object, as a `char`.

        Arguments
        - array: the array
        - index: the index

        Returns
        - the value of the indexed component in the specified array

        Raises
        - NullPointerException: If the specified object is null
        - IllegalArgumentException: If the specified object is not
        an array, or if the indexed element cannot be converted to the
        return type by an identity or widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to the
        length of the specified array

        See
        - Array.get
        """
        ...


    @staticmethod
    def getShort(array: "Object", index: int) -> int:
        """
        Returns the value of the indexed component in the specified
        array object, as a `short`.

        Arguments
        - array: the array
        - index: the index

        Returns
        - the value of the indexed component in the specified array

        Raises
        - NullPointerException: If the specified object is null
        - IllegalArgumentException: If the specified object is not
        an array, or if the indexed element cannot be converted to the
        return type by an identity or widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to the
        length of the specified array

        See
        - Array.get
        """
        ...


    @staticmethod
    def getInt(array: "Object", index: int) -> int:
        """
        Returns the value of the indexed component in the specified
        array object, as an `int`.

        Arguments
        - array: the array
        - index: the index

        Returns
        - the value of the indexed component in the specified array

        Raises
        - NullPointerException: If the specified object is null
        - IllegalArgumentException: If the specified object is not
        an array, or if the indexed element cannot be converted to the
        return type by an identity or widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to the
        length of the specified array

        See
        - Array.get
        """
        ...


    @staticmethod
    def getLong(array: "Object", index: int) -> int:
        """
        Returns the value of the indexed component in the specified
        array object, as a `long`.

        Arguments
        - array: the array
        - index: the index

        Returns
        - the value of the indexed component in the specified array

        Raises
        - NullPointerException: If the specified object is null
        - IllegalArgumentException: If the specified object is not
        an array, or if the indexed element cannot be converted to the
        return type by an identity or widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to the
        length of the specified array

        See
        - Array.get
        """
        ...


    @staticmethod
    def getFloat(array: "Object", index: int) -> float:
        """
        Returns the value of the indexed component in the specified
        array object, as a `float`.

        Arguments
        - array: the array
        - index: the index

        Returns
        - the value of the indexed component in the specified array

        Raises
        - NullPointerException: If the specified object is null
        - IllegalArgumentException: If the specified object is not
        an array, or if the indexed element cannot be converted to the
        return type by an identity or widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to the
        length of the specified array

        See
        - Array.get
        """
        ...


    @staticmethod
    def getDouble(array: "Object", index: int) -> float:
        """
        Returns the value of the indexed component in the specified
        array object, as a `double`.

        Arguments
        - array: the array
        - index: the index

        Returns
        - the value of the indexed component in the specified array

        Raises
        - NullPointerException: If the specified object is null
        - IllegalArgumentException: If the specified object is not
        an array, or if the indexed element cannot be converted to the
        return type by an identity or widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to the
        length of the specified array

        See
        - Array.get
        """
        ...


    @staticmethod
    def set(array: "Object", index: int, value: "Object") -> None:
        """
        Sets the value of the indexed component of the specified array
        object to the specified new value.  The new value is first
        automatically unwrapped if the array has a primitive component
        type.

        Arguments
        - array: the array
        - index: the index into the array
        - value: the new value of the indexed component

        Raises
        - NullPointerException: If the specified object argument
        is null
        - IllegalArgumentException: If the specified object argument
        is not an array, or if the array component type is primitive and
        an unwrapping conversion fails
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to
        the length of the specified array
        """
        ...


    @staticmethod
    def setBoolean(array: "Object", index: int, z: bool) -> None:
        """
        Sets the value of the indexed component of the specified array
        object to the specified `boolean` value.

        Arguments
        - array: the array
        - index: the index into the array
        - z: the new value of the indexed component

        Raises
        - NullPointerException: If the specified object argument
        is null
        - IllegalArgumentException: If the specified object argument
        is not an array, or if the specified value cannot be converted
        to the underlying array's component type by an identity or a
        primitive widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to
        the length of the specified array

        See
        - Array.set
        """
        ...


    @staticmethod
    def setByte(array: "Object", index: int, b: int) -> None:
        """
        Sets the value of the indexed component of the specified array
        object to the specified `byte` value.

        Arguments
        - array: the array
        - index: the index into the array
        - b: the new value of the indexed component

        Raises
        - NullPointerException: If the specified object argument
        is null
        - IllegalArgumentException: If the specified object argument
        is not an array, or if the specified value cannot be converted
        to the underlying array's component type by an identity or a
        primitive widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to
        the length of the specified array

        See
        - Array.set
        """
        ...


    @staticmethod
    def setChar(array: "Object", index: int, c: str) -> None:
        """
        Sets the value of the indexed component of the specified array
        object to the specified `char` value.

        Arguments
        - array: the array
        - index: the index into the array
        - c: the new value of the indexed component

        Raises
        - NullPointerException: If the specified object argument
        is null
        - IllegalArgumentException: If the specified object argument
        is not an array, or if the specified value cannot be converted
        to the underlying array's component type by an identity or a
        primitive widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to
        the length of the specified array

        See
        - Array.set
        """
        ...


    @staticmethod
    def setShort(array: "Object", index: int, s: int) -> None:
        """
        Sets the value of the indexed component of the specified array
        object to the specified `short` value.

        Arguments
        - array: the array
        - index: the index into the array
        - s: the new value of the indexed component

        Raises
        - NullPointerException: If the specified object argument
        is null
        - IllegalArgumentException: If the specified object argument
        is not an array, or if the specified value cannot be converted
        to the underlying array's component type by an identity or a
        primitive widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to
        the length of the specified array

        See
        - Array.set
        """
        ...


    @staticmethod
    def setInt(array: "Object", index: int, i: int) -> None:
        """
        Sets the value of the indexed component of the specified array
        object to the specified `int` value.

        Arguments
        - array: the array
        - index: the index into the array
        - i: the new value of the indexed component

        Raises
        - NullPointerException: If the specified object argument
        is null
        - IllegalArgumentException: If the specified object argument
        is not an array, or if the specified value cannot be converted
        to the underlying array's component type by an identity or a
        primitive widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to
        the length of the specified array

        See
        - Array.set
        """
        ...


    @staticmethod
    def setLong(array: "Object", index: int, l: int) -> None:
        """
        Sets the value of the indexed component of the specified array
        object to the specified `long` value.

        Arguments
        - array: the array
        - index: the index into the array
        - l: the new value of the indexed component

        Raises
        - NullPointerException: If the specified object argument
        is null
        - IllegalArgumentException: If the specified object argument
        is not an array, or if the specified value cannot be converted
        to the underlying array's component type by an identity or a
        primitive widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to
        the length of the specified array

        See
        - Array.set
        """
        ...


    @staticmethod
    def setFloat(array: "Object", index: int, f: float) -> None:
        """
        Sets the value of the indexed component of the specified array
        object to the specified `float` value.

        Arguments
        - array: the array
        - index: the index into the array
        - f: the new value of the indexed component

        Raises
        - NullPointerException: If the specified object argument
        is null
        - IllegalArgumentException: If the specified object argument
        is not an array, or if the specified value cannot be converted
        to the underlying array's component type by an identity or a
        primitive widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to
        the length of the specified array

        See
        - Array.set
        """
        ...


    @staticmethod
    def setDouble(array: "Object", index: int, d: float) -> None:
        """
        Sets the value of the indexed component of the specified array
        object to the specified `double` value.

        Arguments
        - array: the array
        - index: the index into the array
        - d: the new value of the indexed component

        Raises
        - NullPointerException: If the specified object argument
        is null
        - IllegalArgumentException: If the specified object argument
        is not an array, or if the specified value cannot be converted
        to the underlying array's component type by an identity or a
        primitive widening conversion
        - ArrayIndexOutOfBoundsException: If the specified `index`
        argument is negative, or if it is greater than or equal to
        the length of the specified array

        See
        - Array.set
        """
        ...



"""
Python module generated from Java source file com.google.common.graph.GraphBuilder

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import Beta
from com.google.common.base import Optional
from com.google.common.graph import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import DoNotMock
from typing import Any, Callable, Iterable, Tuple


class GraphBuilder(AbstractGraphBuilder):
    """
    A builder for constructing instances of MutableGraph or ImmutableGraph with
    user-defined properties.
    
    A `Graph` built by this class has the following default properties:
    
    
      - does not allow self-loops
      - orders Graph.nodes() in the order in which the elements were added (insertion
          order)
    
    
    `Graph`s built by this class also guarantee that each collection-returning accessor
    returns a **(live) unmodifiable view**; see <a
    href="https://github.com/google/guava/wiki/GraphsExplained#accessor-behavior">the external
    documentation</a> for details.
    
    Examples of use:
    
    ````// Building a mutable graph
    MutableGraph<String> graph = GraphBuilder.undirected().allowsSelfLoops(True).build();
    graph.putEdge("bread", "bread");
    graph.putEdge("chocolate", "peanut butter");
    graph.putEdge("peanut butter", "jelly");
    
    // Building an immutable graph
    ImmutableGraph<String> immutableGraph =
        GraphBuilder.undirected()
            .allowsSelfLoops(True)
            .<String>immutable()
            .putEdge("bread", "bread")
            .putEdge("chocolate", "peanut butter")
            .putEdge("peanut butter", "jelly")
            .build();````
    
    Type `<N>`: The most general node type this builder will support. This is normally `Object`
        unless it is constrained by using a method like .nodeOrder, or the builder is
        constructed based on an existing `Graph` using .from(Graph).

    Author(s)
    - Joshua O'Madadhain

    Since
    - 20.0
    """

    @staticmethod
    def directed() -> "GraphBuilder"["Object"]:
        """
        Returns a GraphBuilder for building directed graphs.
        """
        ...


    @staticmethod
    def undirected() -> "GraphBuilder"["Object"]:
        """
        Returns a GraphBuilder for building undirected graphs.
        """
        ...


    @staticmethod
    def from(graph: "Graph"["N"]) -> "GraphBuilder"["N"]:
        """
        Returns a GraphBuilder initialized with all properties queryable from `graph`.
        
        The "queryable" properties are those that are exposed through the Graph interface,
        such as Graph.isDirected(). Other properties, such as .expectedNodeCount(int),
        are not set in the new builder.
        """
        ...


    def immutable(self) -> "ImmutableGraph.Builder"["N1"]:
        """
        Returns an ImmutableGraph.Builder with the properties of this GraphBuilder.
        
        The returned builder can be used for populating an ImmutableGraph.
        
        Note that the returned builder will always have .incidentEdgeOrder set to ElementOrder.stable(), regardless of the value that was set in this builder.

        Since
        - 28.0
        """
        ...


    def allowsSelfLoops(self, allowsSelfLoops: bool) -> "GraphBuilder"["N"]:
        """
        Specifies whether the graph will allow self-loops (edges that connect a node to itself).
        Attempting to add a self-loop to a graph that does not allow them will throw an UnsupportedOperationException.
        
        The default value is `False`.
        """
        ...


    def expectedNodeCount(self, expectedNodeCount: int) -> "GraphBuilder"["N"]:
        """
        Specifies the expected number of nodes in the graph.

        Raises
        - IllegalArgumentException: if `expectedNodeCount` is negative
        """
        ...


    def nodeOrder(self, nodeOrder: "ElementOrder"["N1"]) -> "GraphBuilder"["N1"]:
        """
        Specifies the order of iteration for the elements of Graph.nodes().
        
        The default value is ElementOrder.insertion() insertion order.
        """
        ...


    def incidentEdgeOrder(self, incidentEdgeOrder: "ElementOrder"["N1"]) -> "GraphBuilder"["N1"]:
        """
        Specifies the order of iteration for the elements of Graph.edges(), Graph.adjacentNodes(Object), Graph.predecessors(Object), Graph.successors(Object) and Graph.incidentEdges(Object).
        
        The default value is ElementOrder.unordered() unordered for mutable graphs. For
        immutable graphs, this value is ignored; they always have a ElementOrder.stable()
        stable order.

        Raises
        - IllegalArgumentException: if `incidentEdgeOrder` is not either `ElementOrder.unordered()` or `ElementOrder.stable()`.

        Since
        - 29.0
        """
        ...


    def build(self) -> "MutableGraph"["N1"]:
        """
        Returns an empty MutableGraph with the properties of this GraphBuilder.
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.TrialSpawner

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import *
from org.bukkit.entity import Entity
from org.bukkit.entity import Player
from org.bukkit.spawner import TrialSpawnerConfiguration
from typing import Any, Callable, Iterable, Tuple


class TrialSpawner(TileState):
    """
    Represents a captured state of a trial spawner.
    """

    def getCooldownLength(self) -> int:
        """
        Gets the length in ticks the spawner will stay in cooldown for.

        Returns
        - the number of ticks
        """
        ...


    def setCooldownLength(self, ticks: int) -> None:
        """
        Sets the length in ticks the spawner will stay in cooldown for.

        Arguments
        - ticks: the number of ticks
        """
        ...


    def getRequiredPlayerRange(self) -> int:
        """
        Get the maximum distance a player can be in order for this
        spawner to be active.
        
        If this value is less than or equal to 0, this spawner is always active
        (given that there are players online).
        
        Default value is 16.

        Returns
        - the maximum distance a player can be in order for this
        spawner to be active.
        """
        ...


    def setRequiredPlayerRange(self, requiredPlayerRange: int) -> None:
        """
        Set the maximum distance a player can be in order for this
        spawner to be active.
        
        Setting this value to less than or equal to 0 will make this spawner
        always active (given that there are players online).

        Arguments
        - requiredPlayerRange: the maximum distance a player can be
        in order for this spawner to be active.
        """
        ...


    def getTrackedPlayers(self) -> Iterable["Player"]:
        """
        Gets the players this spawner is currently tracking.
        
        **Note:** the returned collection is immutable, use
        .startTrackingPlayer(Player) or .stopTrackingPlayer(Player)
        instead.

        Returns
        - a collection of players this spawner is tracking or an empty
                collection if there aren't any
        """
        ...


    def isTrackingPlayer(self, player: "Player") -> bool:
        """
        Checks if this spawner is currently tracking the provided player.

        Arguments
        - player: the player

        Returns
        - True if this spawner is tracking the provided player
        """
        ...


    def startTrackingPlayer(self, player: "Player") -> None:
        """
        Force this spawner to start tracking the provided player.
        
        **Note:** the spawner may decide to stop tracking this player at any given
        time.

        Arguments
        - player: the player
        """
        ...


    def stopTrackingPlayer(self, player: "Player") -> None:
        """
        Force this spawner to stop tracking the provided player.
        
        **Note:** the spawner may decide to start tracking this player again at
        any given time.

        Arguments
        - player: the player
        """
        ...


    def getTrackedEntities(self) -> Iterable["Entity"]:
        """
        Gets a list of entities this spawner is currently tracking.
        
        **Note:** the returned collection is immutable, use
        .startTrackingEntity(Entity) or .stopTrackingEntity(Entity)
        instead.

        Returns
        - a collection of entities this spawner is tracking or an empty
                collection if there aren't any
        """
        ...


    def isTrackingEntity(self, entity: "Entity") -> bool:
        """
        Checks if this spawner is currently tracking the provided entity.

        Arguments
        - entity: the entity

        Returns
        - True if this spawner is tracking the provided entity
        """
        ...


    def startTrackingEntity(self, entity: "Entity") -> None:
        """
        Force this spawner to start tracking the provided entity.
        
        **Note:** the spawner may decide to stop tracking this entity at any given
        time.

        Arguments
        - entity: the entity
        """
        ...


    def stopTrackingEntity(self, entity: "Entity") -> None:
        """
        Force this spawner to stop tracking the provided entity.
        
        **Note:** the spawner may decide to start tracking this entity again at
        any given time.

        Arguments
        - entity: the entity
        """
        ...


    def isOminous(self) -> bool:
        """
        Checks if this spawner is using the ominous
        TrialSpawnerConfiguration.

        Returns
        - True is using the ominous configuration
        """
        ...


    def setOminous(self, ominous: bool) -> None:
        """
        Changes this spawner between the normal and ominous
        TrialSpawnerConfiguration.

        Arguments
        - ominous: True to use the ominous TrialSpawnerConfiguration, False to
                       use the normal one.
        """
        ...


    def getNormalConfiguration(self) -> "TrialSpawnerConfiguration":
        """
        Gets the TrialSpawnerConfiguration used when .isOminous() is
        False.

        Returns
        - the TrialSpawnerConfiguration
        """
        ...


    def getOminousConfiguration(self) -> "TrialSpawnerConfiguration":
        """
        Gets the TrialSpawnerConfiguration used when .isOminous() is
        True.

        Returns
        - the TrialSpawnerConfiguration
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.type.PistonHead

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class PistonHead(TechnicalPiston):
    """
    'short' denotes this piston head is shorter than the usual amount because it
    is currently retracting.
    """

    def isShort(self) -> bool:
        """
        Gets the value of the 'short' property.

        Returns
        - the 'short' value
        """
        ...


    def setShort(self, _short: bool) -> None:
        """
        Sets the value of the 'short' property.

        Arguments
        - _short: the new 'short' value
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.EventHandler

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.event import *
from typing import Any, Callable, Iterable, Tuple


class EventHandler:
    """
    An annotation to mark methods as being event handler methods
    """

    def priority(self) -> "EventPriority":
        """
        Define the priority of the event.
        
        First priority to the last priority executed:
        <ol>
        - LOWEST
        - LOW
        - NORMAL
        - HIGH
        - HIGHEST
        - MONITOR
        </ol>

        Returns
        - the priority
        """
        return EventPriority.NORMAL


    def ignoreCancelled(self) -> bool:
        """
        Define if the handler ignores a cancelled event.
        
        If ignoreCancelled is True and the event is cancelled, the method is
        not called. Otherwise, the method is always called.

        Returns
        - whether cancelled events should be ignored
        """
        return False



"""
Python module generated from Java source file com.google.common.collect.ImmutableList

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import DoNotCall
from com.google.errorprone.annotations import InlineMe
from java.io import InvalidObjectException
from java.io import ObjectInputStream
from java.io import Serializable
from java.util import Arrays
from java.util import Collections
from java.util import Comparator
from java.util import Iterator
from java.util import RandomAccess
from java.util import Spliterator
from java.util.function import Consumer
from java.util.function import UnaryOperator
from java.util.stream import Collector
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ImmutableList(ImmutableCollection, List, RandomAccess):
    """
    A List whose contents will never change, with many other important properties detailed at
    ImmutableCollection.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/ImmutableCollectionsExplained">immutable collections</a>.

    Author(s)
    - Kevin Bourrillion

    See
    - ImmutableSet

    Since
    - 2.0
    """

    @staticmethod
    def toImmutableList() -> "Collector"["E", Any, "ImmutableList"["E"]]:
        """
        Returns a `Collector` that accumulates the input elements into a new `ImmutableList`, in encounter order.

        Since
        - 21.0
        """
        ...


    @staticmethod
    def of() -> "ImmutableList"["E"]:
        ...


    @staticmethod
    def of(e1: "E") -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing a single element. This list behaves and performs
        comparably to Collections.singletonList, but will not accept a null element. It is
        preferable mainly for consistency and maintainability of your code.

        Raises
        - NullPointerException: if the element is null
        """
        ...


    @staticmethod
    def of(e1: "E", e2: "E") -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order.

        Raises
        - NullPointerException: if any element is null
        """
        ...


    @staticmethod
    def of(e1: "E", e2: "E", e3: "E") -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order.

        Raises
        - NullPointerException: if any element is null
        """
        ...


    @staticmethod
    def of(e1: "E", e2: "E", e3: "E", e4: "E") -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order.

        Raises
        - NullPointerException: if any element is null
        """
        ...


    @staticmethod
    def of(e1: "E", e2: "E", e3: "E", e4: "E", e5: "E") -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order.

        Raises
        - NullPointerException: if any element is null
        """
        ...


    @staticmethod
    def of(e1: "E", e2: "E", e3: "E", e4: "E", e5: "E", e6: "E") -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order.

        Raises
        - NullPointerException: if any element is null
        """
        ...


    @staticmethod
    def of(e1: "E", e2: "E", e3: "E", e4: "E", e5: "E", e6: "E", e7: "E") -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order.

        Raises
        - NullPointerException: if any element is null
        """
        ...


    @staticmethod
    def of(e1: "E", e2: "E", e3: "E", e4: "E", e5: "E", e6: "E", e7: "E", e8: "E") -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order.

        Raises
        - NullPointerException: if any element is null
        """
        ...


    @staticmethod
    def of(e1: "E", e2: "E", e3: "E", e4: "E", e5: "E", e6: "E", e7: "E", e8: "E", e9: "E") -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order.

        Raises
        - NullPointerException: if any element is null
        """
        ...


    @staticmethod
    def of(e1: "E", e2: "E", e3: "E", e4: "E", e5: "E", e6: "E", e7: "E", e8: "E", e9: "E", e10: "E") -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order.

        Raises
        - NullPointerException: if any element is null
        """
        ...


    @staticmethod
    def of(e1: "E", e2: "E", e3: "E", e4: "E", e5: "E", e6: "E", e7: "E", e8: "E", e9: "E", e10: "E", e11: "E") -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order.

        Raises
        - NullPointerException: if any element is null
        """
        ...


    @staticmethod
    def of(e1: "E", e2: "E", e3: "E", e4: "E", e5: "E", e6: "E", e7: "E", e8: "E", e9: "E", e10: "E", e11: "E", e12: "E", *others: Tuple["E", ...]) -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order.
        
        The array `others` must not be longer than `Integer.MAX_VALUE - 12`.

        Raises
        - NullPointerException: if any element is null

        Since
        - 3.0 (source-compatible since 2.0)
        """
        ...


    @staticmethod
    def copyOf(elements: Iterable["E"]) -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order. If `elements` is a
        Collection, this method behaves exactly as .copyOf(Collection); otherwise, it
        behaves exactly as `copyOf(elements.iterator()`.

        Raises
        - NullPointerException: if `elements` contains a null element
        """
        ...


    @staticmethod
    def copyOf(elements: Iterable["E"]) -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order.
        
        Despite the method name, this method attempts to avoid actually copying the data when it is
        safe to do so. The exact circumstances under which a copy will or will not be performed are
        undocumented and subject to change.
        
        Note that if `list` is a `List<String>`, then `ImmutableList.copyOf(list)`
        returns an `ImmutableList<String>` containing each of the strings in `list`, while
        `ImmutableList.of(list)` returns an `ImmutableList<List<String>>` containing one
        element (the given list itself).
        
        This method is safe to use even when `elements` is a synchronized or concurrent
        collection that is currently being modified by another thread.

        Raises
        - NullPointerException: if `elements` contains a null element
        """
        ...


    @staticmethod
    def copyOf(elements: Iterator["E"]) -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order.

        Raises
        - NullPointerException: if `elements` contains a null element
        """
        ...


    @staticmethod
    def copyOf(elements: list["E"]) -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in order.

        Raises
        - NullPointerException: if `elements` contains a null element

        Since
        - 3.0
        """
        ...


    @staticmethod
    def sortedCopyOf(elements: Iterable["E"]) -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, sorted according to their natural
        order. The sorting algorithm used is stable, so elements that compare as equal will stay in the
        order in which they appear in the input.
        
        If your data has no duplicates, or you wish to deduplicate elements, use `ImmutableSortedSet.copyOf(elements)`; if you want a `List` you can use its `asList()` view.
        
        **Java 8+ users:** If you want to convert a java.util.stream.Stream to a sorted
        `ImmutableList`, use `stream.sorted().collect(toImmutableList())`.

        Raises
        - NullPointerException: if any element in the input is null

        Since
        - 21.0
        """
        ...


    @staticmethod
    def sortedCopyOf(comparator: "Comparator"["E"], elements: Iterable["E"]) -> "ImmutableList"["E"]:
        """
        Returns an immutable list containing the given elements, in sorted order relative to the
        specified comparator. The sorting algorithm used is stable, so elements that compare as equal
        will stay in the order in which they appear in the input.
        
        If your data has no duplicates, or you wish to deduplicate elements, use `ImmutableSortedSet.copyOf(comparator, elements)`; if you want a `List` you can use its
        `asList()` view.
        
        **Java 8+ users:** If you want to convert a java.util.stream.Stream to a sorted
        `ImmutableList`, use `stream.sorted(comparator).collect(toImmutableList())`.

        Raises
        - NullPointerException: if any element in the input is null

        Since
        - 21.0
        """
        ...


    def iterator(self) -> "UnmodifiableIterator"["E"]:
        ...


    def listIterator(self) -> "UnmodifiableListIterator"["E"]:
        ...


    def listIterator(self, index: int) -> "UnmodifiableListIterator"["E"]:
        ...


    def forEach(self, consumer: "Consumer"["E"]) -> None:
        ...


    def indexOf(self, object: "Object") -> int:
        ...


    def lastIndexOf(self, object: "Object") -> int:
        ...


    def contains(self, object: "Object") -> bool:
        ...


    def subList(self, fromIndex: int, toIndex: int) -> "ImmutableList"["E"]:
        """
        Returns an immutable list of the elements between the specified `fromIndex`, inclusive,
        and `toIndex`, exclusive. (If `fromIndex` and `toIndex` are equal, the empty
        immutable list is returned.)
        
        **Note:** in almost all circumstances, the returned ImmutableList retains a
        strong reference to `this`, which may prevent the original list from being garbage
        collected. If you want the original list to be eligible for garbage collection, you should
        create and use a copy of the sub list (e.g., `ImmutableList.copyOf(originalList.subList(...))`).
        """
        ...


    def addAll(self, index: int, newElements: Iterable["E"]) -> bool:
        """
        Guaranteed to throw an exception and leave the list unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def set(self, index: int, element: "E") -> "E":
        """
        Guaranteed to throw an exception and leave the list unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def add(self, index: int, element: "E") -> None:
        """
        Guaranteed to throw an exception and leave the list unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def remove(self, index: int) -> "E":
        """
        Guaranteed to throw an exception and leave the list unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def replaceAll(self, operator: "UnaryOperator"["E"]) -> None:
        """
        Guaranteed to throw an exception and leave the list unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def sort(self, c: "Comparator"["E"]) -> None:
        """
        Guaranteed to throw an exception and leave the list unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def asList(self) -> "ImmutableList"["E"]:
        """
        Returns this list instance.

        Since
        - 2.0

        Deprecated
        - There is no reason to use this; it always returns `this`.
        """
        ...


    def spliterator(self) -> "Spliterator"["E"]:
        ...


    def reverse(self) -> "ImmutableList"["E"]:
        """
        Returns a view of this immutable list in reverse order. For example, `ImmutableList.of(1,
        2, 3).reverse()` is equivalent to `ImmutableList.of(3, 2, 1)`.

        Returns
        - a view of this immutable list in reverse order

        Since
        - 7.0
        """
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    @staticmethod
    def builder() -> "Builder"["E"]:
        """
        Returns a new builder. The generated builder is equivalent to the builder created by the Builder constructor.
        """
        ...


    @staticmethod
    def builderWithExpectedSize(expectedSize: int) -> "Builder"["E"]:
        """
        Returns a new builder, expecting the specified number of elements to be added.
        
        If `expectedSize` is exactly the number of elements added to the builder before Builder.build is called, the builder is likely to perform better than an unsized .builder() would have.
        
        It is not specified if any performance benefits apply if `expectedSize` is close to,
        but not exactly, the number of elements added to the builder.

        Since
        - 23.1
        """
        ...


    class Builder(Builder):
        """
        A builder for creating immutable list instances, especially `public static final` lists
        ("constant lists"). Example:
        
        ````public static final ImmutableList<Color> GOOGLE_COLORS
            = new ImmutableList.Builder<Color>()
                .addAll(WEBSAFE_COLORS)
                .add(new Color(0, 191, 255))
                .build();````
        
        Elements appear in the resulting list in the same order they were added to the builder.
        
        Builder instances can be reused; it is safe to call .build multiple times to build
        multiple lists in series. Each new list contains all the elements of the ones created before
        it.

        Since
        - 2.0
        """

        def __init__(self):
            """
            Creates a new builder. The returned builder is equivalent to the builder generated by ImmutableList.builder.
            """
            ...


        def add(self, element: "E") -> "Builder"["E"]:
            """
            Adds `element` to the `ImmutableList`.

            Arguments
            - element: the element to add

            Returns
            - this `Builder` object

            Raises
            - NullPointerException: if `element` is null
            """
            ...


        def add(self, *elements: Tuple["E", ...]) -> "Builder"["E"]:
            """
            Adds each element of `elements` to the `ImmutableList`.

            Arguments
            - elements: the `Iterable` to add to the `ImmutableList`

            Returns
            - this `Builder` object

            Raises
            - NullPointerException: if `elements` is null or contains a null element
            """
            ...


        def addAll(self, elements: Iterable["E"]) -> "Builder"["E"]:
            """
            Adds each element of `elements` to the `ImmutableList`.

            Arguments
            - elements: the `Iterable` to add to the `ImmutableList`

            Returns
            - this `Builder` object

            Raises
            - NullPointerException: if `elements` is null or contains a null element
            """
            ...


        def addAll(self, elements: Iterator["E"]) -> "Builder"["E"]:
            """
            Adds each element of `elements` to the `ImmutableList`.

            Arguments
            - elements: the `Iterator` to add to the `ImmutableList`

            Returns
            - this `Builder` object

            Raises
            - NullPointerException: if `elements` is null or contains a null element
            """
            ...


        def build(self) -> "ImmutableList"["E"]:
            """
            Returns a newly-created `ImmutableList` based on the contents of the `Builder`.
            """
            ...



"""
Python module generated from Java source file com.google.common.base.Function

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import *
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Function(Function):
    """
    Legacy version of java.util.function.Function java.util.function.Function.
    
    The Functions class provides common functions and related utilities.
    
    As this interface extends `java.util.function.Function`, an instance of this type can be
    used as a `java.util.function.Function` directly. To use a `java.util.function.Function` in a context where a `com.google.common.base.Function` is
    needed, use `function::apply`.
    
    This interface is now a legacy type. Use `java.util.function.Function` (or the
    appropriate primitive specialization such as `ToIntFunction`) instead whenever possible.
    Otherwise, at least reduce *explicit* dependencies on this type by using lambda expressions
    or method references instead of classes, leaving your code easier to migrate in the future.
    
    See the Guava User Guide article on <a
    href="https://github.com/google/guava/wiki/FunctionalExplained">the use of `Function`</a>.

    Author(s)
    - Kevin Bourrillion

    Since
    - 2.0
    """

    def apply(self, input: "F") -> "T":
        ...


    def equals(self, object: "Object") -> bool:
        """
        *May* return `True` if `object` is a `Function` that behaves identically
        to this function.
        
        **Warning: do not depend** on the behavior of this method.
        
        Historically, `Function` instances in this library have implemented this method to
        recognize certain cases where distinct `Function` instances would in fact behave
        identically. However, as code migrates to `java.util.function`, that behavior will
        disappear. It is best not to depend on it.
        """
        ...



"""
Python module generated from Java source file java.lang.reflect.AccessibleObject

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.invoke import MethodHandle
from java.lang.ref import WeakReference
from java.lang.reflect import *
from java.security import AccessController
from jdk.internal.access import SharedSecrets
from jdk.internal.misc import VM
from jdk.internal.reflect import CallerSensitive
from jdk.internal.reflect import Reflection
from jdk.internal.reflect import ReflectionFactory
from sun.security.action import GetPropertyAction
from sun.security.util import SecurityConstants
from typing import Any, Callable, Iterable, Tuple


class AccessibleObject(AnnotatedElement):
    """
    The `AccessibleObject` class is the base class for `Field`,
    `Method`, and `Constructor` objects (known as *reflected
    objects*). It provides the ability to flag a reflected object as
    suppressing checks for Java language access control when it is used. This
    permits sophisticated applications with sufficient privilege, such as Java
    Object Serialization or other persistence mechanisms, to manipulate objects
    in a manner that would normally be prohibited.
    
     Java language access control prevents use of private members outside
    their top-level class; package access members outside their package; protected members
    outside their package or subclasses; and public members outside their
    module unless they are declared in an Module.isExported(String,Module)
    exported package and the user Module.canRead reads their module. By
    default, Java language access control is enforced (with one variation) when
    `Field`s, `Method`s, or `Constructor`s are used to get or
    set fields, to invoke methods, or to create and initialize new instances of
    classes, respectively. Every reflected object checks that the code using it
    is in an appropriate class, package, or module. The check when invoked by
    <a href="/../specs/jni/index.html">JNI code</a> with no Java
    class on the stack only succeeds if the member and the declaring class are
    public, and the class is in a package that is exported to all modules. 
    
     The one variation from Java language access control is that the checks
    by reflected objects assume readability. That is, the module containing
    the use of a reflected object is assumed to read the module in which
    the underlying field, method, or constructor is declared. 
    
     Whether the checks for Java language access control can be suppressed
    (and thus, whether access can be enabled) depends on whether the reflected
    object corresponds to a member in an exported or open package
    (see .setAccessible(boolean)). 

    Since
    - 1.2

    Unknown Tags
    - 6.6 Access Control
    - 9
    """

    @staticmethod
    def setAccessible(array: list["AccessibleObject"], flag: bool) -> None:
        """
        Convenience method to set the `accessible` flag for an
        array of reflected objects with a single security check (for efficiency).
        
         This method may be used to enable access to all reflected objects in
        the array when access to each reflected object can be enabled as
        specified by .setAccessible(boolean) setAccessible(boolean). 
        
        If there is a security manager, its
        `checkPermission` method is first called with a
        `ReflectPermission("suppressAccessChecks")` permission.
        
        A `SecurityException` is also thrown if any of the elements of
        the input `array` is a java.lang.reflect.Constructor
        object for the class `java.lang.Class` and `flag` is True.

        Arguments
        - array: the array of AccessibleObjects
        - flag: the new value for the `accessible` flag
                     in each object

        Raises
        - InaccessibleObjectException: if access cannot be enabled for all
                objects in the array
        - SecurityException: if the request is denied by the security manager
                or an element in the array is a constructor for `java.lang.Class`

        See
        - ReflectPermission

        Unknown Tags
        - 9
        """
        ...


    def setAccessible(self, flag: bool) -> None:
        """
        Set the `accessible` flag for this reflected object to
        the indicated boolean value.  A value of `True` indicates that
        the reflected object should suppress checks for Java language access
        control when it is used. A value of `False` indicates that
        the reflected object should enforce checks for Java language access
        control when it is used, with the variation noted in the class description.
        
         This method may be used by a caller in class `C` to enable
        access to a Member member of Member.getDeclaringClass()
        declaring class `D` if any of the following hold: 
        
        
            -  `C` and `D` are in the same module. 
        
            -  The member is `public` and `D` is `public` in
            a package that the module containing `D` Module.isExported(String,Module) exports to at least the module
            containing `C`. 
        
            -  The member is `protected` `static`, `D` is
            `public` in a package that the module containing `D`
            exports to at least the module containing `C`, and `C`
            is a subclass of `D`. 
        
            -  `D` is in a package that the module containing `D`
            Module.isOpen(String,Module) opens to at least the module
            containing `C`.
            All packages in unnamed and open modules are open to all modules and
            so this method always succeeds when `D` is in an unnamed or
            open module. 
        
        
         This method cannot be used to enable access to private members,
        members with default (package) access, protected instance members, or
        protected constructors when the declaring class is in a different module
        to the caller and the package containing the declaring class is not open
        to the caller's module. 
        
         This method cannot be used to enable Field.set *write*
        access to a *non-modifiable* final field.  The following fields
        are non-modifiable:
        
        - static final fields declared in any class or interface
        - final fields declared in a Class.isHidden() hidden class
        - final fields declared in a Class.isRecord() record
        
         The `accessible` flag when `True` suppresses Java language access
        control checks to only enable Field.get *read* access to
        these non-modifiable final fields.
        
         If there is a security manager, its
        `checkPermission` method is first called with a
        `ReflectPermission("suppressAccessChecks")` permission.

        Arguments
        - flag: the new value for the `accessible` flag

        Raises
        - InaccessibleObjectException: if access cannot be enabled
        - SecurityException: if the request is denied by the security manager

        See
        - java.lang.invoke.MethodHandles.privateLookupIn

        Unknown Tags
        - 9
        """
        ...


    def trySetAccessible(self) -> bool:
        """
        Set the `accessible` flag for this reflected object to `True`
        if possible. This method sets the `accessible` flag, as if by
        invoking .setAccessible(boolean) setAccessible(True), and returns
        the possibly-updated value for the `accessible` flag. If access
        cannot be enabled, i.e. the checks or Java language access control cannot
        be suppressed, this method returns `False` (as opposed to `setAccessible(True)` throwing `InaccessibleObjectException` when
        it fails).
        
         This method is a no-op if the `accessible` flag for
        this reflected object is `True`.
        
         For example, a caller can invoke `trySetAccessible`
        on a `Method` object for a private instance method
        `p.T::privateMethod` to suppress the checks for Java language access
        control when the `Method` is invoked.
        If `p.T` class is in a different module to the caller and
        package `p` is open to at least the caller's module,
        the code below successfully sets the `accessible` flag
        to `True`.
        
        ```
        `p.T obj = ....;  // instance of p.T
            :
            Method m = p.T.class.getDeclaredMethod("privateMethod");
            if (m.trySetAccessible()) {
                m.invoke(obj);` else {
                // package p is not opened to the caller to access private member of T
                ...
            }
        }```
        
         If there is a security manager, its `checkPermission` method
        is first called with a `ReflectPermission("suppressAccessChecks")`
        permission. 

        Returns
        - `True` if the `accessible` flag is set to `True`;
                `False` if access cannot be enabled.

        Raises
        - SecurityException: if the request is denied by the security manager

        See
        - java.lang.invoke.MethodHandles.privateLookupIn

        Since
        - 9
        """
        ...


    def isAccessible(self) -> bool:
        """
        Get the value of the `accessible` flag for this reflected object.

        Returns
        - the value of the object's `accessible` flag

        Deprecated
        - This method is deprecated because its name hints that it checks
        if the reflected object is accessible when it actually indicates
        if the checks for Java language access control are suppressed.
        This method may return `False` on a reflected object that is
        accessible to the caller. To test if this reflected object is accessible,
        it should use .canAccess(Object).

        Unknown Tags
        - 9
        """
        ...


    def canAccess(self, obj: "Object") -> bool:
        """
        Test if the caller can access this reflected object. If this reflected
        object corresponds to an instance method or field then this method tests
        if the caller can access the given `obj` with the reflected object.
        For instance methods or fields then the `obj` argument must be an
        instance of the Member.getDeclaringClass() declaring class. For
        static members and constructors then `obj` must be `null`.
        
         This method returns `True` if the `accessible` flag
        is set to `True`, i.e. the checks for Java language access control
        are suppressed, or if the caller can access the member as
        specified in <cite>The Java Language Specification</cite>,
        with the variation noted in the class description. 

        Arguments
        - obj: an instance object of the declaring class of this reflected
                   object if it is an instance method or field

        Returns
        - `True` if the caller can access this reflected object.

        Raises
        - IllegalArgumentException: 
                -  if this reflected object is a static member or constructor and
                     the given `obj` is non-`null`, or 
                -  if this reflected object is an instance method or field
                     and the given `obj` is `null` or of type
                     that is not a subclass of the Member.getDeclaringClass()
                     declaring class of the member.
                

        See
        - .setAccessible(boolean)

        Since
        - 9

        Unknown Tags
        - 6.6 Access Control
        """
        ...


    def getAnnotation(self, annotationClass: type["T"]) -> "T":
        """
        
        
         Note that any annotation returned by this method is a
        declaration annotation.

        Raises
        - NullPointerException: 

        Since
        - 1.5

        Unknown Tags
        - The default implementation throws UnsupportedOperationException; subclasses should override this method.
        """
        ...


    def isAnnotationPresent(self, annotationClass: type["Annotation"]) -> bool:
        """
        Raises
        - NullPointerException: 

        Since
        - 1.5
        """
        ...


    def getAnnotationsByType(self, annotationClass: type["T"]) -> list["T"]:
        """
        
        
         Note that any annotations returned by this method are
        declaration annotations.

        Raises
        - NullPointerException: 

        Since
        - 1.8

        Unknown Tags
        - The default implementation throws UnsupportedOperationException; subclasses should override this method.
        """
        ...


    def getAnnotations(self) -> list["Annotation"]:
        """
        
        
         Note that any annotations returned by this method are
        declaration annotations.

        Since
        - 1.5
        """
        ...


    def getDeclaredAnnotation(self, annotationClass: type["T"]) -> "T":
        """
        
        
         Note that any annotation returned by this method is a
        declaration annotation.

        Raises
        - NullPointerException: 

        Since
        - 1.8
        """
        ...


    def getDeclaredAnnotationsByType(self, annotationClass: type["T"]) -> list["T"]:
        """
        
        
         Note that any annotations returned by this method are
        declaration annotations.

        Raises
        - NullPointerException: 

        Since
        - 1.8
        """
        ...


    def getDeclaredAnnotations(self) -> list["Annotation"]:
        """
        
        
         Note that any annotations returned by this method are
        declaration annotations.

        Since
        - 1.5

        Unknown Tags
        - The default implementation throws UnsupportedOperationException; subclasses should override this method.
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.Ordering

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import Function
from com.google.common.collect import *
from java.util import Arrays
from java.util import Collections
from java.util import Comparator
from java.util import Iterator
from java.util import NoSuchElementException
from java.util import SortedMap
from java.util import SortedSet
from java.util.concurrent import ConcurrentMap
from java.util.concurrent.atomic import AtomicInteger
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import NonNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Ordering(Comparator):
    """
    A comparator, with additional methods to support common operations. This is an "enriched" version
    of `Comparator` for pre-Java-8 users, in the same sense that FluentIterable is an
    enriched Iterable for pre-Java-8 users.
    
    <h3>Three types of methods</h3>
    
    Like other fluent types, there are three types of methods present: methods for *acquiring*,
    *chaining*, and *using*.
    
    <h4>Acquiring</h4>
    
    The common ways to get an instance of `Ordering` are:
    
    
      - Subclass it and implement .compare instead of implementing Comparator
          directly
      - Pass a *pre-existing* Comparator instance to .from(Comparator)
      - Use the natural ordering, Ordering.natural
    
    
    <h4>Chaining</h4>
    
    Then you can use the *chaining* methods to get an altered version of that `Ordering`, including:
    
    
      - .reverse
      - .compound(Comparator)
      - .onResultOf(Function)
      - .nullsFirst / .nullsLast
    
    
    <h4>Using</h4>
    
    Finally, use the resulting `Ordering` anywhere a Comparator is required, or use
    any of its special operations, such as:
    
    
      - .immutableSortedCopy
      - .isOrdered / .isStrictlyOrdered
      - .min / .max
    
    
    <h3>Understanding complex orderings</h3>
    
    Complex chained orderings like the following example can be challenging to understand.
    
    ````Ordering<Foo> ordering =
        Ordering.natural()
            .nullsFirst()
            .onResultOf(getBarFunction)
            .nullsLast();````
    
    Note that each chaining method returns a new ordering instance which is backed by the previous
    instance, but has the chance to act on values *before* handing off to that backing instance.
    As a result, it usually helps to read chained ordering expressions *backwards*. For example,
    when `compare` is called on the above ordering:
    
    <ol>
      - First, if only one `Foo` is null, that null value is treated as *greater*
      - Next, non-null `Foo` values are passed to `getBarFunction` (we will be
          comparing `Bar` values from now on)
      - Next, if only one `Bar` is null, that null value is treated as *lesser*
      - Finally, natural ordering is used (i.e. the result of `Bar.compareTo(Bar)` is
          returned)
    </ol>
    
    Alas, .reverse is a little different. As you read backwards through a chain and
    encounter a call to `reverse`, continue working backwards until a result is determined, and
    then reverse that result.
    
    <h3>Additional notes</h3>
    
    Except as noted, the orderings returned by the factory methods of this class are serializable
    if and only if the provided instances that back them are. For example, if `ordering` and
    `function` can themselves be serialized, then `ordering.onResultOf(function)` can as
    well.
    
    <h3>Java 8+ users</h3>
    
    If you are using Java 8+, this class is now obsolete. Most of its functionality is now
    provided by java.util.stream.Stream Stream and by Comparator itself, and the rest
    can now be found as static methods in our new Comparators class. See each method below
    for further instructions. Whenever possible, you should change any references of type `Ordering` to be of type `Comparator` instead. However, at this time we have no plan to
    *deprecate* this class.
    
    Many replacements involve adopting `Stream`, and these changes can sometimes make your
    code verbose. Whenever following this advice, you should check whether `Stream` could be
    adopted more comprehensively in your code; the end result may be quite a bit simpler.
    
    <h3>See also</h3>
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/OrderingExplained">`Ordering`</a>.

    Author(s)
    - Kevin Bourrillion

    Since
    - 2.0
    """

    @staticmethod
    def natural() -> "Ordering"["C"]:
        """
        Returns a serializable ordering that uses the natural order of the values. The ordering throws
        a NullPointerException when passed a null parameter.
        
        The type specification is `<C extends Comparable>`, instead of the technically correct
        `<C extends Comparable<? super C>>`, to support legacy types from before Java 5.
        
        **Java 8+ users:** use Comparator.naturalOrder instead.
        """
        ...


    @staticmethod
    def from(comparator: "Comparator"["T"]) -> "Ordering"["T"]:
        """
        Returns an ordering based on an *existing* comparator instance. Note that it is
        unnecessary to create a *new* anonymous inner class implementing `Comparator` just
        to pass it in here. Instead, simply subclass `Ordering` and implement its `compare`
        method directly.
        
        The returned object is serializable if `comparator` is serializable.
        
        **Java 8+ users:** this class is now obsolete as explained in the class documentation, so
        there is no need to use this method.

        Arguments
        - comparator: the comparator that defines the order

        Returns
        - comparator itself if it is already an `Ordering`; otherwise an ordering that
            wraps that comparator
        """
        ...


    @staticmethod
    def from(ordering: "Ordering"["T"]) -> "Ordering"["T"]:
        """
        Simply returns its argument.

        Deprecated
        - no need to use this
        """
        ...


    @staticmethod
    def explicit(valuesInOrder: list["T"]) -> "Ordering"["T"]:
        ...


    @staticmethod
    def explicit(leastValue: "T", *remainingValuesInOrder: Tuple["T", ...]) -> "Ordering"["T"]:
        ...


    @staticmethod
    def allEqual() -> "Ordering"["Object"]:
        """
        Returns an ordering which treats all values as equal, indicating "no ordering." Passing this
        ordering to any *stable* sort algorithm results in no change to the order of elements.
        Note especially that .sortedCopy and .immutableSortedCopy are stable, and in
        the returned instance these are implemented by simply copying the source list.
        
        Example:
        
        ````Ordering.allEqual().nullsLast().sortedCopy(
            asList(t, null, e, s, null, t, null))````
        
        Assuming `t`, `e` and `s` are non-null, this returns `[t, e, s, t,
        null, null, null]` regardless of the True comparison order of those three values (which might
        not even implement Comparable at all).
        
        **Warning:** by definition, this comparator is not *consistent with equals* (as
        defined Comparator here). Avoid its use in APIs, such as TreeSet.TreeSet(Comparator), where such consistency is expected.
        
        The returned comparator is serializable.
        
        **Java 8+ users:** Use the lambda expression `(a, b) -> 0` instead (in certain
        cases you may need to cast that to `Comparator<YourType>`).

        Since
        - 13.0
        """
        ...


    @staticmethod
    def usingToString() -> "Ordering"["Object"]:
        """
        Returns an ordering that compares objects by the natural ordering of their string
        representations as returned by `toString()`. It does not support null values.
        
        The comparator is serializable.
        
        **Java 8+ users:** Use `Comparator.comparing(Object::toString)` instead.
        """
        ...


    @staticmethod
    def arbitrary() -> "Ordering"["Object"]:
        ...


    def reverse(self) -> "Ordering"["S"]:
        ...


    def nullsFirst(self) -> "Ordering"["S"]:
        ...


    def nullsLast(self) -> "Ordering"["S"]:
        ...


    def onResultOf(self, function: "Function"["F", "T"]) -> "Ordering"["F"]:
        """
        Returns a new ordering on `F` which orders elements by first applying a function to them,
        then comparing those results using `this`. For example, to compare objects by their
        string forms, in a case-insensitive manner, use:
        
        ````Ordering.from(String.CASE_INSENSITIVE_ORDER)
            .onResultOf(Functions.toStringFunction())````
        
        **Java 8+ users:** Use `Comparator.comparing(function, thisComparator)` instead
        (you can omit the comparator if it is the natural order).
        """
        ...


    def compound(self, secondaryComparator: "Comparator"["U"]) -> "Ordering"["U"]:
        """
        Returns an ordering which first uses the ordering `this`, but which in the event of a
        "tie", then delegates to `secondaryComparator`. For example, to sort a bug list first by
        status and second by priority, you might use `byStatus.compound(byPriority)`. For a
        compound ordering with three or more components, simply chain multiple calls to this method.
        
        An ordering produced by this method, or a chain of calls to this method, is equivalent to
        one created using Ordering.compound(Iterable) on the same component comparators.
        
        The returned object is serializable if this object and `secondaryComparator` are both
        serializable.
        
        **Java 8+ users:** Use `thisComparator.thenComparing(secondaryComparator)` instead.
        Depending on what `secondaryComparator` is, one of the other overloads of `thenComparing` may be even more useful.
        """
        ...


    @staticmethod
    def compound(comparators: Iterable["Comparator"["T"]]) -> "Ordering"["T"]:
        """
        Returns an ordering which tries each given comparator in order until a non-zero result is
        found, returning that result, and returning zero only if all comparators return zero. The
        returned ordering is based on the state of the `comparators` iterable at the time it was
        provided to this method.
        
        The returned ordering is equivalent to that produced using `Ordering.from(comp1).compound(comp2).compound(comp3) . . .`.
        
        The returned object is serializable if each of the `comparators` is serializable.
        
        **Warning:** Supplying an argument with undefined iteration order, such as a HashSet, will produce non-deterministic results.
        
        **Java 8+ users:** Use a chain of calls to Comparator.thenComparing(Comparator),
        or `comparatorCollection.stream().reduce(Comparator::thenComparing).get()` (if the
        collection might be empty, also provide a default comparator as the `identity` parameter
        to `reduce`).

        Arguments
        - comparators: the comparators to try in order
        """
        ...


    def lexicographical(self) -> "Ordering"[Iterable["S"]]:
        """
        Returns a new ordering which sorts iterables by comparing corresponding elements pairwise until
        a nonzero result is found; imposes "dictionary order". If the end of one iterable is reached,
        but not the other, the shorter iterable is considered to be less than the longer one. For
        example, a lexicographical natural ordering over integers considers `[] < [1] < [1, 1] <
        [1, 2] < [2]`.
        
        Note that `ordering.lexicographical().reverse()` is not equivalent to `ordering.reverse().lexicographical()` (consider how each would order `[1]` and `[1,
        1]`).
        
        **Java 8+ users:** Use Comparators.lexicographical(Comparator) instead.

        Since
        - 2.0
        """
        ...


    def compare(self, left: "T", right: "T") -> int:
        ...


    def min(self, iterator: Iterator["E"]) -> "E":
        """
        Returns the least of the specified values according to this ordering. If there are multiple
        least values, the first of those is returned. The iterator will be left exhausted: its `hasNext()` method will return `False`.
        
        **Java 8+ users:** Use `Streams.stream(iterator).min(thisComparator).get()` instead
        (but note that it does not guarantee which tied minimum element is returned).

        Arguments
        - iterator: the iterator whose minimum element is to be determined

        Raises
        - NoSuchElementException: if `iterator` is empty
        - ClassCastException: if the parameters are not *mutually comparable* under this
            ordering.

        Since
        - 11.0
        """
        ...


    def min(self, iterable: Iterable["E"]) -> "E":
        """
        Returns the least of the specified values according to this ordering. If there are multiple
        least values, the first of those is returned.
        
        **Java 8+ users:** If `iterable` is a Collection, use `Collections.min(collection, thisComparator)` instead. Otherwise, use `Streams.stream(iterable).min(thisComparator).get()` instead. Note that these alternatives do
        not guarantee which tied minimum element is returned.

        Arguments
        - iterable: the iterable whose minimum element is to be determined

        Raises
        - NoSuchElementException: if `iterable` is empty
        - ClassCastException: if the parameters are not *mutually comparable* under this
            ordering.
        """
        ...


    def min(self, a: "E", b: "E") -> "E":
        """
        Returns the lesser of the two values according to this ordering. If the values compare as 0,
        the first is returned.
        
        **Implementation note:** this method is invoked by the default implementations of the
        other `min` overloads, so overriding it will affect their behavior.
        
        **Note:** Consider using `Comparators.min(a, b, thisComparator)` instead. If `thisComparator` is Ordering.natural, then use `Comparators.min(a, b)`.

        Arguments
        - a: value to compare, returned if less than or equal to b.
        - b: value to compare.

        Raises
        - ClassCastException: if the parameters are not *mutually comparable* under this
            ordering.
        """
        ...


    def min(self, a: "E", b: "E", c: "E", *rest: Tuple["E", ...]) -> "E":
        """
        Returns the least of the specified values according to this ordering. If there are multiple
        least values, the first of those is returned.
        
        **Java 8+ users:** Use `Collections.min(Arrays.asList(a, b, c...), thisComparator)`
        instead (but note that it does not guarantee which tied minimum element is returned).

        Arguments
        - a: value to compare, returned if less than or equal to the rest.
        - b: value to compare
        - c: value to compare
        - rest: values to compare

        Raises
        - ClassCastException: if the parameters are not *mutually comparable* under this
            ordering.
        """
        ...


    def max(self, iterator: Iterator["E"]) -> "E":
        """
        Returns the greatest of the specified values according to this ordering. If there are multiple
        greatest values, the first of those is returned. The iterator will be left exhausted: its
        `hasNext()` method will return `False`.
        
        **Java 8+ users:** Use `Streams.stream(iterator).max(thisComparator).get()` instead
        (but note that it does not guarantee which tied maximum element is returned).

        Arguments
        - iterator: the iterator whose maximum element is to be determined

        Raises
        - NoSuchElementException: if `iterator` is empty
        - ClassCastException: if the parameters are not *mutually comparable* under this
            ordering.

        Since
        - 11.0
        """
        ...


    def max(self, iterable: Iterable["E"]) -> "E":
        """
        Returns the greatest of the specified values according to this ordering. If there are multiple
        greatest values, the first of those is returned.
        
        **Java 8+ users:** If `iterable` is a Collection, use `Collections.max(collection, thisComparator)` instead. Otherwise, use `Streams.stream(iterable).max(thisComparator).get()` instead. Note that these alternatives do
        not guarantee which tied maximum element is returned.

        Arguments
        - iterable: the iterable whose maximum element is to be determined

        Raises
        - NoSuchElementException: if `iterable` is empty
        - ClassCastException: if the parameters are not *mutually comparable* under this
            ordering.
        """
        ...


    def max(self, a: "E", b: "E") -> "E":
        """
        Returns the greater of the two values according to this ordering. If the values compare as 0,
        the first is returned.
        
        **Implementation note:** this method is invoked by the default implementations of the
        other `max` overloads, so overriding it will affect their behavior.
        
        **Note:** Consider using `Comparators.max(a, b, thisComparator)` instead. If `thisComparator` is Ordering.natural, then use `Comparators.max(a, b)`.

        Arguments
        - a: value to compare, returned if greater than or equal to b.
        - b: value to compare.

        Raises
        - ClassCastException: if the parameters are not *mutually comparable* under this
            ordering.
        """
        ...


    def max(self, a: "E", b: "E", c: "E", *rest: Tuple["E", ...]) -> "E":
        """
        Returns the greatest of the specified values according to this ordering. If there are multiple
        greatest values, the first of those is returned.
        
        **Java 8+ users:** Use `Collections.max(Arrays.asList(a, b, c...), thisComparator)`
        instead (but note that it does not guarantee which tied maximum element is returned).

        Arguments
        - a: value to compare, returned if greater than or equal to the rest.
        - b: value to compare
        - c: value to compare
        - rest: values to compare

        Raises
        - ClassCastException: if the parameters are not *mutually comparable* under this
            ordering.
        """
        ...


    def leastOf(self, iterable: Iterable["E"], k: int) -> list["E"]:
        """
        Returns the `k` least elements of the given iterable according to this ordering, in order
        from least to greatest. If there are fewer than `k` elements present, all will be
        included.
        
        The implementation does not necessarily use a *stable* sorting algorithm; when multiple
        elements are equivalent, it is undefined which will come first.
        
        **Java 8+ users:** Use `Streams.stream(iterable).collect(Comparators.least(k,
        thisComparator))` instead.

        Returns
        - an immutable `RandomAccess` list of the `k` least elements in ascending
            order

        Raises
        - IllegalArgumentException: if `k` is negative

        Since
        - 8.0
        """
        ...


    def leastOf(self, iterator: Iterator["E"], k: int) -> list["E"]:
        """
        Returns the `k` least elements from the given iterator according to this ordering, in
        order from least to greatest. If there are fewer than `k` elements present, all will be
        included.
        
        The implementation does not necessarily use a *stable* sorting algorithm; when multiple
        elements are equivalent, it is undefined which will come first.
        
        **Java 8+ users:** Use `Streams.stream(iterator).collect(Comparators.least(k,
        thisComparator))` instead.

        Returns
        - an immutable `RandomAccess` list of the `k` least elements in ascending
            order

        Raises
        - IllegalArgumentException: if `k` is negative

        Since
        - 14.0
        """
        ...


    def greatestOf(self, iterable: Iterable["E"], k: int) -> list["E"]:
        """
        Returns the `k` greatest elements of the given iterable according to this ordering, in
        order from greatest to least. If there are fewer than `k` elements present, all will be
        included.
        
        The implementation does not necessarily use a *stable* sorting algorithm; when multiple
        elements are equivalent, it is undefined which will come first.
        
        **Java 8+ users:** Use `Streams.stream(iterable).collect(Comparators.greatest(k,
        thisComparator))` instead.

        Returns
        - an immutable `RandomAccess` list of the `k` greatest elements in
            *descending order*

        Raises
        - IllegalArgumentException: if `k` is negative

        Since
        - 8.0
        """
        ...


    def greatestOf(self, iterator: Iterator["E"], k: int) -> list["E"]:
        """
        Returns the `k` greatest elements from the given iterator according to this ordering, in
        order from greatest to least. If there are fewer than `k` elements present, all will be
        included.
        
        The implementation does not necessarily use a *stable* sorting algorithm; when multiple
        elements are equivalent, it is undefined which will come first.
        
        **Java 8+ users:** Use `Streams.stream(iterator).collect(Comparators.greatest(k,
        thisComparator))` instead.

        Returns
        - an immutable `RandomAccess` list of the `k` greatest elements in
            *descending order*

        Raises
        - IllegalArgumentException: if `k` is negative

        Since
        - 14.0
        """
        ...


    def sortedCopy(self, elements: Iterable["E"]) -> list["E"]:
        ...


    def immutableSortedCopy(self, elements: Iterable["E"]) -> "ImmutableList"["E"]:
        ...


    def isOrdered(self, iterable: Iterable["T"]) -> bool:
        """
        Returns `True` if each element in `iterable` after the first is greater than or
        equal to the element that preceded it, according to this ordering. Note that this is always
        True when the iterable has fewer than two elements.
        
        **Java 8+ users:** Use the equivalent Comparators.isInOrder(Iterable, Comparator)
        instead, since the rest of `Ordering` is mostly obsolete (as explained in the class
        documentation).
        """
        ...


    def isStrictlyOrdered(self, iterable: Iterable["T"]) -> bool:
        """
        Returns `True` if each element in `iterable` after the first is *strictly*
        greater than the element that preceded it, according to this ordering. Note that this is always
        True when the iterable has fewer than two elements.
        
        **Java 8+ users:** Use the equivalent Comparators.isInStrictOrder(Iterable,
        Comparator) instead, since the rest of `Ordering` is mostly obsolete (as explained in
        the class documentation).
        """
        ...


    def binarySearch(self, sortedList: list["T"], key: "T") -> int:
        """
        Collections.binarySearch(List, Object, Comparator) Searches `sortedList` for
        `key` using the binary search algorithm. The list must be sorted using this ordering.

        Arguments
        - sortedList: the list to be searched
        - key: the key to be searched for

        Deprecated
        - Use Collections.binarySearch(List, Object, Comparator) directly.
        """
        ...



"""
Python module generated from Java source file java.util.function.Function

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Objects
from java.util.function import *
from typing import Any, Callable, Iterable, Tuple


class Function:
    """
    Represents a function that accepts one argument and produces a result.
    
    This is a <a href="package-summary.html">functional interface</a>
    whose functional method is .apply(Object).
    
    Type `<T>`: the type of the input to the function
    
    Type `<R>`: the type of the result of the function

    Since
    - 1.8
    """

    def apply(self, t: "T") -> "R":
        """
        Applies this function to the given argument.

        Arguments
        - t: the function argument

        Returns
        - the function result
        """
        ...


    def compose(self, before: "Function"["V", "T"]) -> "Function"["V", "R"]:
        """
        Returns a composed function that first applies the `before`
        function to its input, and then applies this function to the result.
        If evaluation of either function throws an exception, it is relayed to
        the caller of the composed function.
        
        Type `<V>`: the type of input to the `before` function, and to the
                  composed function

        Arguments
        - before: the function to apply before this function is applied

        Returns
        - a composed function that first applies the `before`
        function and then applies this function

        Raises
        - NullPointerException: if before is null

        See
        - .andThen(Function)
        """
        ...


    def andThen(self, after: "Function"["R", "V"]) -> "Function"["T", "V"]:
        """
        Returns a composed function that first applies this function to
        its input, and then applies the `after` function to the result.
        If evaluation of either function throws an exception, it is relayed to
        the caller of the composed function.
        
        Type `<V>`: the type of output of the `after` function, and of the
                  composed function

        Arguments
        - after: the function to apply after this function is applied

        Returns
        - a composed function that first applies this function and then
        applies the `after` function

        Raises
        - NullPointerException: if after is null

        See
        - .compose(Function)
        """
        ...


    @staticmethod
    def identity() -> "Function"["T", "T"]:
        """
        Returns a function that always returns its input argument.
        
        Type `<T>`: the type of the input and output objects to the function

        Returns
        - a function that always returns its input argument
        """
        ...



"""
Python module generated from Java source file org.bukkit.Translatable

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class Translatable:
    """
    Represents an object with a text representation that can be translated by the
    Minecraft client.
    """

    def getTranslationKey(self) -> str:
        """
        Get the translation key, suitable for use in a translation component.

        Returns
        - the translation key
        """
        ...



"""
Python module generated from Java source file org.spigotmc.event.player.PlayerSpawnLocationEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Location
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import PlayerEvent
from org.spigotmc.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerSpawnLocationEvent(PlayerEvent):
    """
    Called when player is about to spawn in a world after joining the server.
    """

    def __init__(self, who: "Player", spawnLocation: "Location"):
        ...


    def getSpawnLocation(self) -> "Location":
        """
        Gets player's spawn location.
        If the player Player.hasPlayedBefore(), it's going to default to the location inside player.dat file.
        For new players, the default spawn location is spawn of the main Bukkit world.

        Returns
        - the spawn location
        """
        ...


    def setSpawnLocation(self, location: "Location") -> None:
        """
        Sets player's spawn location.

        Arguments
        - location: the spawn location
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.Rotation

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class Rotation(Enum):
    """
    An enum to specify a rotation based orientation, like that on a clock.
    
    It represents how something is viewed, as opposed to cardinal directions.
    """

    NONE = 0
    """
    No rotation
    """
    CLOCKWISE_45 = 1
    """
    Rotated clockwise by 45 degrees
    """
    CLOCKWISE = 2
    """
    Rotated clockwise by 90 degrees
    """
    CLOCKWISE_135 = 3
    """
    Rotated clockwise by 135 degrees
    """
    FLIPPED = 4
    """
    Flipped upside-down, a 180 degree rotation
    """
    FLIPPED_45 = 5
    """
    Flipped upside-down + 45 degree rotation
    """
    COUNTER_CLOCKWISE = 6
    """
    Rotated counter-clockwise by 90 degrees
    """
    COUNTER_CLOCKWISE_45 = 7
    """
    Rotated counter-clockwise by 45 degrees
    """


    def rotateClockwise(self) -> "Rotation":
        """
        Rotate clockwise by 90 degrees.

        Returns
        - the relative rotation
        """
        ...


    def rotateCounterClockwise(self) -> "Rotation":
        """
        Rotate counter-clockwise by 90 degrees.

        Returns
        - the relative rotation
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.Endermite

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Endermite(Monster):

    def isPlayerSpawned(self) -> bool:
        """
        Gets whether this Endermite was spawned by a player.
        
        An Endermite spawned by a player will be attacked by nearby Enderman.

        Returns
        - player spawned status

        Deprecated
        - this functionality no longer exists
        """
        ...


    def setPlayerSpawned(self, playerSpawned: bool) -> None:
        """
        Sets whether this Endermite was spawned by a player.
        
        An Endermite spawned by a player will be attacked by nearby Enderman.

        Arguments
        - playerSpawned: player spawned status

        Deprecated
        - this functionality no longer exists
        """
        ...



"""
Python module generated from Java source file java.io.DataOutputStream

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from typing import Any, Callable, Iterable, Tuple


class DataOutputStream(FilterOutputStream, DataOutput):
    """
    A data output stream lets an application write primitive Java data
    types to an output stream in a portable way. An application can
    then use a data input stream to read the data back in.
    
    A DataOutputStream is not safe for use by multiple concurrent
    threads. If a DataOutputStream is to be used by more than one
    thread then access to the data output stream should be controlled
    by appropriate synchronization.

    See
    - java.io.DataInputStream

    Since
    - 1.0
    """

    def __init__(self, out: "OutputStream"):
        """
        Creates a new data output stream to write data to the specified
        underlying output stream. The counter `written` is
        set to zero.

        Arguments
        - out: the underlying output stream, to be saved for later
                       use.

        See
        - java.io.FilterOutputStream.out
        """
        ...


    def write(self, b: int) -> None:
        """
        Writes the specified byte (the low eight bits of the argument
        `b`) to the underlying output stream. If no exception
        is thrown, the counter `written` is incremented by
        `1`.
        
        Implements the `write` method of `OutputStream`.

        Arguments
        - b: the `byte` to be written.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterOutputStream.out
        """
        ...


    def write(self, b: list[int], off: int, len: int) -> None:
        """
        Writes `len` bytes from the specified byte array
        starting at offset `off` to the underlying output stream.
        If no exception is thrown, the counter `written` is
        incremented by `len`.

        Arguments
        - b: the data.
        - off: the start offset in the data.
        - len: the number of bytes to write.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterOutputStream.out
        """
        ...


    def flush(self) -> None:
        """
        Flushes this data output stream. This forces any buffered output
        bytes to be written out to the stream.
        
        The `flush` method of `DataOutputStream`
        calls the `flush` method of its underlying output stream.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.OutputStream.flush()
        """
        ...


    def writeBoolean(self, v: bool) -> None:
        """
        Writes a `boolean` to the underlying output stream as
        a 1-byte value. The value `True` is written out as the
        value `(byte)1`; the value `False` is
        written out as the value `(byte)0`. If no exception is
        thrown, the counter `written` is incremented by
        `1`.

        Arguments
        - v: a `boolean` value to be written.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterOutputStream.out
        """
        ...


    def writeByte(self, v: int) -> None:
        """
        Writes out a `byte` to the underlying output stream as
        a 1-byte value. If no exception is thrown, the counter
        `written` is incremented by `1`.

        Arguments
        - v: a `byte` value to be written.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterOutputStream.out
        """
        ...


    def writeShort(self, v: int) -> None:
        """
        Writes a `short` to the underlying output stream as two
        bytes, high byte first. If no exception is thrown, the counter
        `written` is incremented by `2`.

        Arguments
        - v: a `short` to be written.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterOutputStream.out
        """
        ...


    def writeChar(self, v: int) -> None:
        """
        Writes a `char` to the underlying output stream as a
        2-byte value, high byte first. If no exception is thrown, the
        counter `written` is incremented by `2`.

        Arguments
        - v: a `char` value to be written.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterOutputStream.out
        """
        ...


    def writeInt(self, v: int) -> None:
        """
        Writes an `int` to the underlying output stream as four
        bytes, high byte first. If no exception is thrown, the counter
        `written` is incremented by `4`.

        Arguments
        - v: an `int` to be written.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterOutputStream.out
        """
        ...


    def writeLong(self, v: int) -> None:
        """
        Writes a `long` to the underlying output stream as eight
        bytes, high byte first. In no exception is thrown, the counter
        `written` is incremented by `8`.

        Arguments
        - v: a `long` to be written.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterOutputStream.out
        """
        ...


    def writeFloat(self, v: float) -> None:
        """
        Converts the float argument to an `int` using the
        `floatToIntBits` method in class `Float`,
        and then writes that `int` value to the underlying
        output stream as a 4-byte quantity, high byte first. If no
        exception is thrown, the counter `written` is
        incremented by `4`.

        Arguments
        - v: a `float` value to be written.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.lang.Float.floatToIntBits(float)
        """
        ...


    def writeDouble(self, v: float) -> None:
        """
        Converts the double argument to a `long` using the
        `doubleToLongBits` method in class `Double`,
        and then writes that `long` value to the underlying
        output stream as an 8-byte quantity, high byte first. If no
        exception is thrown, the counter `written` is
        incremented by `8`.

        Arguments
        - v: a `double` value to be written.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.lang.Double.doubleToLongBits(double)
        """
        ...


    def writeBytes(self, s: str) -> None:
        """
        Writes out the string to the underlying output stream as a
        sequence of bytes. Each character in the string is written out, in
        sequence, by discarding its high eight bits. If no exception is
        thrown, the counter `written` is incremented by the
        length of `s`.

        Arguments
        - s: a string of bytes to be written.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterOutputStream.out
        """
        ...


    def writeChars(self, s: str) -> None:
        """
        Writes a string to the underlying output stream as a sequence of
        characters. Each character is written to the data output stream as
        if by the `writeChar` method. If no exception is
        thrown, the counter `written` is incremented by twice
        the length of `s`.

        Arguments
        - s: a `String` value to be written.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.FilterOutputStream.out
        """
        ...


    def writeUTF(self, str: str) -> None:
        """
        Writes a string to the underlying output stream using
        <a href="DataInput.html#modified-utf-8">modified UTF-8</a>
        encoding in a machine-independent manner.
        
        First, two bytes are written to the output stream as if by the
        `writeShort` method giving the number of bytes to
        follow. This value is the number of bytes actually written out,
        not the length of the string. Following the length, each character
        of the string is output, in sequence, using the modified UTF-8 encoding
        for the character. If no exception is thrown, the counter
        `written` is incremented by the total number of
        bytes written to the output stream. This will be at least two
        plus the length of `str`, and at most two plus
        thrice the length of `str`.

        Arguments
        - str: a string to be written.

        Raises
        - UTFDataFormatException: if the modified UTF-8 encoding of
                    `str` would exceed 65535 bytes in length
        - IOException: if some other I/O error occurs.

        See
        - .writeChars(String)
        """
        ...


    def size(self) -> int:
        """
        Returns the current value of the counter `written`,
        the number of bytes written to this data output stream so far.
        If the counter overflows, it will be wrapped to Integer.MAX_VALUE.

        Returns
        - the value of the `written` field.

        See
        - java.io.DataOutputStream.written
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.EntityTargetLivingEntityEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Entity
from org.bukkit.entity import LivingEntity
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class EntityTargetLivingEntityEvent(EntityTargetEvent):
    """
    Called when an Entity targets a LivingEntity and can only target
    LivingEntity's.
    """

    def __init__(self, entity: "Entity", target: "LivingEntity", reason: "TargetReason"):
        ...


    def getTarget(self) -> "LivingEntity":
        ...


    def setTarget(self, target: "Entity") -> None:
        """
        Set the Entity that you want the mob to target.
        
        It is possible to be null, null will cause the entity to be
        target-less.
        
        Must be a LivingEntity, or null.

        Arguments
        - target: The entity to target
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerJoinEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerJoinEvent(PlayerEvent):
    """
    Called when a player joins a server
    """

    def __init__(self, playerJoined: "Player", joinMessage: str):
        ...


    def getJoinMessage(self) -> str:
        """
        Gets the join message to send to all online players

        Returns
        - string join message. Can be null
        """
        ...


    def setJoinMessage(self, joinMessage: str) -> None:
        """
        Sets the join message to send to all online players

        Arguments
        - joinMessage: join message. If null, no message will be sent
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.collect.Streams

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import Beta
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.common.math import LongMath
from com.google.errorprone.annotations import InlineMe
from com.google.errorprone.annotations import InlineMeValidationDisabled
from java.util import ArrayDeque
from java.util import Deque
from java.util import Iterator
from java.util import OptionalDouble
from java.util import OptionalInt
from java.util import OptionalLong
from java.util import PrimitiveIterator
from java.util import Spliterator
from java.util.function import BiConsumer
from java.util.function import BiFunction
from java.util.function import Consumer
from java.util.function import DoubleConsumer
from java.util.function import IntConsumer
from java.util.function import LongConsumer
from java.util.stream import BaseStream
from java.util.stream import DoubleStream
from java.util.stream import IntStream
from java.util.stream import LongStream
from java.util.stream import Stream
from java.util.stream import StreamSupport
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Streams:
    """
    Static utility methods related to `Stream` instances.

    Since
    - 21.0
    """

    @staticmethod
    def stream(iterable: Iterable["T"]) -> "Stream"["T"]:
        """
        Returns a sequential Stream of the contents of `iterable`, delegating to Collection.stream if possible.
        """
        ...


    @staticmethod
    def stream(collection: Iterable["T"]) -> "Stream"["T"]:
        """
        Returns Collection.stream.

        Deprecated
        - There is no reason to use this; just invoke `collection.stream()` directly.
        """
        ...


    @staticmethod
    def stream(iterator: Iterator["T"]) -> "Stream"["T"]:
        """
        Returns a sequential Stream of the remaining contents of `iterator`. Do not use
        `iterator` directly after passing it to this method.
        """
        ...


    @staticmethod
    def stream(optional: "com.google.common.base.Optional"["T"]) -> "Stream"["T"]:
        """
        If a value is present in `optional`, returns a stream containing only that element,
        otherwise returns an empty stream.
        """
        ...


    @staticmethod
    def stream(optional: "java.util.Optional"["T"]) -> "Stream"["T"]:
        """
        If a value is present in `optional`, returns a stream containing only that element,
        otherwise returns an empty stream.
        
        **Java 9 users:** use `optional.stream()` instead.
        """
        ...


    @staticmethod
    def stream(optional: "OptionalInt") -> "IntStream":
        """
        If a value is present in `optional`, returns a stream containing only that element,
        otherwise returns an empty stream.
        
        **Java 9 users:** use `optional.stream()` instead.
        """
        ...


    @staticmethod
    def stream(optional: "OptionalLong") -> "LongStream":
        """
        If a value is present in `optional`, returns a stream containing only that element,
        otherwise returns an empty stream.
        
        **Java 9 users:** use `optional.stream()` instead.
        """
        ...


    @staticmethod
    def stream(optional: "OptionalDouble") -> "DoubleStream":
        """
        If a value is present in `optional`, returns a stream containing only that element,
        otherwise returns an empty stream.
        
        **Java 9 users:** use `optional.stream()` instead.
        """
        ...


    @staticmethod
    def concat(*streams: Tuple["Stream"["T"], ...]) -> "Stream"["T"]:
        """
        Returns a Stream containing the elements of the first stream, followed by the elements
        of the second stream, and so on.
        
        This is equivalent to `Stream.of(streams).flatMap(stream -> stream)`, but the returned
        stream may perform better.

        See
        - Stream.concat(Stream, Stream)
        """
        ...


    @staticmethod
    def concat(*streams: Tuple["IntStream", ...]) -> "IntStream":
        """
        Returns an IntStream containing the elements of the first stream, followed by the
        elements of the second stream, and so on.
        
        This is equivalent to `Stream.of(streams).flatMapToInt(stream -> stream)`, but the
        returned stream may perform better.

        See
        - IntStream.concat(IntStream, IntStream)
        """
        ...


    @staticmethod
    def concat(*streams: Tuple["LongStream", ...]) -> "LongStream":
        """
        Returns a LongStream containing the elements of the first stream, followed by the
        elements of the second stream, and so on.
        
        This is equivalent to `Stream.of(streams).flatMapToLong(stream -> stream)`, but the
        returned stream may perform better.

        See
        - LongStream.concat(LongStream, LongStream)
        """
        ...


    @staticmethod
    def concat(*streams: Tuple["DoubleStream", ...]) -> "DoubleStream":
        """
        Returns a DoubleStream containing the elements of the first stream, followed by the
        elements of the second stream, and so on.
        
        This is equivalent to `Stream.of(streams).flatMapToDouble(stream -> stream)`, but the
        returned stream may perform better.

        See
        - DoubleStream.concat(DoubleStream, DoubleStream)
        """
        ...


    @staticmethod
    def zip(streamA: "Stream"["A"], streamB: "Stream"["B"], function: "BiFunction"["A", "B", "R"]) -> "Stream"["R"]:
        """
        Returns a stream in which each element is the result of passing the corresponding element of
        each of `streamA` and `streamB` to `function`.
        
        For example:
        
        ````Streams.zip(
          Stream.of("foo1", "foo2", "foo3"),
          Stream.of("bar1", "bar2"),
          (arg1, arg2) -> arg1 + ":" + arg2)````
        
        will return `Stream.of("foo1:bar1", "foo2:bar2")`.
        
        The resulting stream will only be as long as the shorter of the two input streams; if one
        stream is longer, its extra elements will be ignored.
        
        Note that if you are calling Stream.forEach on the resulting stream, you might want
        to consider using .forEachPair instead of this method.
        
        **Performance note:** The resulting stream is not <a
        href="http://gee.cs.oswego.edu/dl/html/StreamParallelGuidance.html">efficiently splittable</a>.
        This may harm parallel performance.
        """
        ...


    @staticmethod
    def forEachPair(streamA: "Stream"["A"], streamB: "Stream"["B"], consumer: "BiConsumer"["A", "B"]) -> None:
        """
        Invokes `consumer` once for each pair of *corresponding* elements in `streamA`
        and `streamB`. If one stream is longer than the other, the extra elements are silently
        ignored. Elements passed to the consumer are guaranteed to come from the same position in their
        respective source streams. For example:
        
        ````Streams.forEachPair(
          Stream.of("foo1", "foo2", "foo3"),
          Stream.of("bar1", "bar2"),
          (arg1, arg2) -> System.out.println(arg1 + ":" + arg2)````
        
        will print:
        
        ````foo1:bar1
        foo2:bar2````
        
        **Warning:** If either supplied stream is a parallel stream, the same correspondence
        between elements will be made, but the order in which those pairs of elements are passed to the
        consumer is *not* defined.
        
        Note that many usages of this method can be replaced with simpler calls to .zip.
        This method behaves equivalently to .zip zipping the stream elements into
        temporary pair objects and then using Stream.forEach on that stream.

        Since
        - 22.0
        """
        ...


    @staticmethod
    def mapWithIndex(stream: "Stream"["T"], function: "FunctionWithIndex"["T", "R"]) -> "Stream"["R"]:
        """
        Returns a stream consisting of the results of applying the given function to the elements of
        `stream` and their indices in the stream. For example,
        
        ````mapWithIndex(
            Stream.of("a", "b", "c"),
            (e, index) -> index + ":" + e)````
        
        would return `Stream.of("0:a", "1:b", "2:c")`.
        
        The resulting stream is <a
        href="http://gee.cs.oswego.edu/dl/html/StreamParallelGuidance.html">efficiently splittable</a>
        if and only if `stream` was efficiently splittable and its underlying spliterator
        reported Spliterator.SUBSIZED. This is generally the case if the underlying stream
        comes from a data structure supporting efficient indexed random access, typically an array or
        list.
        
        The order of the resulting stream is defined if and only if the order of the original stream
        was defined.
        """
        ...


    @staticmethod
    def mapWithIndex(stream: "IntStream", function: "IntFunctionWithIndex"["R"]) -> "Stream"["R"]:
        """
        Returns a stream consisting of the results of applying the given function to the elements of
        `stream` and their indexes in the stream. For example,
        
        ````mapWithIndex(
            IntStream.of(10, 11, 12),
            (e, index) -> index + ":" + e)````
        
        ...would return `Stream.of("0:10", "1:11", "2:12")`.
        
        The resulting stream is <a
        href="http://gee.cs.oswego.edu/dl/html/StreamParallelGuidance.html">efficiently splittable</a>
        if and only if `stream` was efficiently splittable and its underlying spliterator
        reported Spliterator.SUBSIZED. This is generally the case if the underlying stream
        comes from a data structure supporting efficient indexed random access, typically an array or
        list.
        
        The order of the resulting stream is defined if and only if the order of the original stream
        was defined.
        """
        ...


    @staticmethod
    def mapWithIndex(stream: "LongStream", function: "LongFunctionWithIndex"["R"]) -> "Stream"["R"]:
        """
        Returns a stream consisting of the results of applying the given function to the elements of
        `stream` and their indexes in the stream. For example,
        
        ````mapWithIndex(
            LongStream.of(10, 11, 12),
            (e, index) -> index + ":" + e)````
        
        ...would return `Stream.of("0:10", "1:11", "2:12")`.
        
        The resulting stream is <a
        href="http://gee.cs.oswego.edu/dl/html/StreamParallelGuidance.html">efficiently splittable</a>
        if and only if `stream` was efficiently splittable and its underlying spliterator
        reported Spliterator.SUBSIZED. This is generally the case if the underlying stream
        comes from a data structure supporting efficient indexed random access, typically an array or
        list.
        
        The order of the resulting stream is defined if and only if the order of the original stream
        was defined.
        """
        ...


    @staticmethod
    def mapWithIndex(stream: "DoubleStream", function: "DoubleFunctionWithIndex"["R"]) -> "Stream"["R"]:
        """
        Returns a stream consisting of the results of applying the given function to the elements of
        `stream` and their indexes in the stream. For example,
        
        ````mapWithIndex(
            DoubleStream.of(0.0, 1.0, 2.0)
            (e, index) -> index + ":" + e)````
        
        ...would return `Stream.of("0:0.0", "1:1.0", "2:2.0")`.
        
        The resulting stream is <a
        href="http://gee.cs.oswego.edu/dl/html/StreamParallelGuidance.html">efficiently splittable</a>
        if and only if `stream` was efficiently splittable and its underlying spliterator
        reported Spliterator.SUBSIZED. This is generally the case if the underlying stream
        comes from a data structure supporting efficient indexed random access, typically an array or
        list.
        
        The order of the resulting stream is defined if and only if the order of the original stream
        was defined.
        """
        ...


    @staticmethod
    def findLast(stream: "Stream"["T"]) -> "java.util.Optional"["T"]:
        ...


    @staticmethod
    def findLast(stream: "IntStream") -> "OptionalInt":
        """
        Returns the last element of the specified stream, or OptionalInt.empty if the stream is
        empty.
        
        Equivalent to `stream.reduce((a, b) -> b)`, but may perform significantly better. This
        method's runtime will be between O(log n) and O(n), performing better on <a
        href="http://gee.cs.oswego.edu/dl/html/StreamParallelGuidance.html">efficiently splittable</a>
        streams.

        Raises
        - NullPointerException: if the last element of the stream is null

        See
        - IntStream.findFirst()
        """
        ...


    @staticmethod
    def findLast(stream: "LongStream") -> "OptionalLong":
        """
        Returns the last element of the specified stream, or OptionalLong.empty if the stream
        is empty.
        
        Equivalent to `stream.reduce((a, b) -> b)`, but may perform significantly better. This
        method's runtime will be between O(log n) and O(n), performing better on <a
        href="http://gee.cs.oswego.edu/dl/html/StreamParallelGuidance.html">efficiently splittable</a>
        streams.

        Raises
        - NullPointerException: if the last element of the stream is null

        See
        - LongStream.findFirst()
        """
        ...


    @staticmethod
    def findLast(stream: "DoubleStream") -> "OptionalDouble":
        """
        Returns the last element of the specified stream, or OptionalDouble.empty if the stream
        is empty.
        
        Equivalent to `stream.reduce((a, b) -> b)`, but may perform significantly better. This
        method's runtime will be between O(log n) and O(n), performing better on <a
        href="http://gee.cs.oswego.edu/dl/html/StreamParallelGuidance.html">efficiently splittable</a>
        streams.

        Raises
        - NullPointerException: if the last element of the stream is null

        See
        - DoubleStream.findFirst()
        """
        ...


    class FunctionWithIndex:
        """
        An analogue of java.util.function.Function also accepting an index.
        
        This interface is only intended for use by callers of .mapWithIndex(Stream,
        FunctionWithIndex).

        Since
        - 21.0
        """

        def apply(self, from: "T", index: int) -> "R":
            """
            Applies this function to the given argument and its index within a stream.
            """
            ...


    class IntFunctionWithIndex:
        """
        An analogue of java.util.function.IntFunction also accepting an index.
        
        This interface is only intended for use by callers of .mapWithIndex(IntStream,
        IntFunctionWithIndex).

        Since
        - 21.0
        """

        def apply(self, from: int, index: int) -> "R":
            """
            Applies this function to the given argument and its index within a stream.
            """
            ...


    class LongFunctionWithIndex:
        """
        An analogue of java.util.function.LongFunction also accepting an index.
        
        This interface is only intended for use by callers of .mapWithIndex(LongStream,
        LongFunctionWithIndex).

        Since
        - 21.0
        """

        def apply(self, from: int, index: int) -> "R":
            """
            Applies this function to the given argument and its index within a stream.
            """
            ...


    class DoubleFunctionWithIndex:
        """
        An analogue of java.util.function.DoubleFunction also accepting an index.
        
        This interface is only intended for use by callers of .mapWithIndex(DoubleStream,
        DoubleFunctionWithIndex).

        Since
        - 21.0
        """

        def apply(self, from: float, index: int) -> "R":
            """
            Applies this function to the given argument and its index within a stream.
            """
            ...



"""
Python module generated from Java source file org.bukkit.event.server.ServerCommandEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.command import CommandSender
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.server import *
from typing import Any, Callable, Iterable, Tuple


class ServerCommandEvent(ServerEvent, Cancellable):
    """
    This event is called when a command is run by a non-player. It is
    called early in the command handling process, and modifications in this
    event (via .setCommand(String)) will be shown in the behavior.
    
    Many plugins will have **no use for this event**, and you should
    attempt to avoid using it if it is not necessary.
    
    Some examples of valid uses for this event are:
    
    - Logging executed commands to a separate file
    - Variable substitution. For example, replacing `${ip:Steve}`
        with the connection IP of the player named Steve, or simulating the
        `@a` and `@p` decorators used by Command Blocks
        for plugins that do not handle it.
    - Conditionally blocking commands belonging to other plugins.
    - Per-sender command aliases. For example, after the console runs the
        command `/calias cr gamemode creative`, the next time they
        run `/cr`, it gets replaced into
        `/gamemode creative`. (Global command aliases should be
        done by registering the alias.)
    
    
    Examples of incorrect uses are:
    
    - Using this event to run command logic
    
    
    If the event is cancelled, processing of the command will halt.
    
    The state of whether or not there is a slash (`/`) at the
    beginning of the message should be preserved. If a slash is added or
    removed, unexpected behavior may result.
    """

    def __init__(self, sender: "CommandSender", command: str):
        ...


    def getCommand(self) -> str:
        """
        Gets the command that the user is attempting to execute from the
        console

        Returns
        - Command the user is attempting to execute
        """
        ...


    def setCommand(self, message: str) -> None:
        """
        Sets the command that the server will execute

        Arguments
        - message: New message that the server will execute
        """
        ...


    def getSender(self) -> "CommandSender":
        """
        Get the command sender.

        Returns
        - The sender
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...
